/* hst_hist -- plot histograms & distribution curves using GMT 5 (classic mode of GMT 6)
		array size of tick[] increased   031921
*/

#include <string.h>
#include <math.h>
#include "histo.h"

extern char File[NF], Filet[NF], Filep[NF];
extern unsigned int Opt;
extern int N, Nfrq;
extern double *X, Xmax, Xmin, dX, Ymax, Ymin, *Frq, Mxl, Sxl, Frcn[NFRC], Frcl[NFRC];

static char map[50], range[100], tick[300], sbar[100];

void hst_hist(mx, sx)
double mx, sx;
{
	FILE *fp;
	int i;
	double x0, x0c, dxc, dxg, ddxg, dyg, ddyg, dum1, dum2;

	/* set up points of distribution curves */

	x0 = Xmin + dX/2.;   dxc = (Xmax - Xmin)/NFRC;   x0c = Xmin + dxc/2.;
	if (Opt & NORM_BIT){
		for (i=0; i<NFRC; i++)
			Frcn[i] = N*normdist((x0c + dxc*i - mx)/sx)*dX/sx;
	}
	if (Opt & LOGNORM_BIT){
		for (i=0; i<NFRC; i++){
			if ((dum1 = x0c + dxc*i) <= 0.) Frcl[i] = 0.;
			else {
				dum2 = (log(dum1) - Mxl)/Sxl;
				Frcl[i] = N*normdist(dum2)*dX/(Sxl*dum1);
			}
		}
	}

	/* set up axis parameters & initialize GMT */

	sprintf(map, " -JX%.2f/%.2f", MAPWIDTH, MAPHEIGHT);
	g_xmax_dx(Xmax, Xmin, &dxg, &ddxg);
	g_xmax_dx(Ymax, Ymin, &dyg, &ddyg);
	sprintf(range, " -R%.4g/%.4g/%.4g/%.4g", Xmin, Xmax, Ymin, Ymax);
	sprintf(tick," -BWeSn+t\"Histogram of %s (N=%d)\" -Bxa%.4gf%.4g+lData -Bya%.4gf%.4g+lFrequency",File,N,dxg,ddxg,dyg,ddyg);
	sys_call(2, "gmt gmtset", GSET);

	/* plot a histogram */

	if ((fp=fopen(Filet,"w")) == NULL) error("Error of fopen at hst_hist!");
	for (i=0; i<Nfrq; i++)
		fprintf(fp, "%.4g %.4g\n", x0 + dX*i, Frq[i]);
	fclose(fp);
	sprintf(sbar, " -Sb%.4g", MAPWIDTH*dX/(Xmax - Xmin));
	if (!(Opt & NORM_BIT) && !(Opt & LOGNORM_BIT))
		plot_hist();
	else if ((Opt & NORM_BIT) && !(Opt & LOGNORM_BIT)){
		plot_histK();
		create_curve(x0c, dxc, Frcn);
		plot_curven();
	}
	else if (!(Opt & NORM_BIT) && (Opt & LOGNORM_BIT)){
		plot_histK();
		create_curve(x0c, dxc, Frcl);
		plot_curvel();
	}
	else {
		plot_histK();
		create_curve(x0c, dxc, Frcn);
		plot_curvenK();
		create_curve(x0c, dxc, Frcl);
		plot_curveld();
	}
}

void plot_hist()
{ sys_call(8,"gmt psxy ",Filet,map,range,tick,sbar," -Wthinnest -Glightgray -X5.0 -Y10.0 -P >",Filep); }

void plot_histK()
{ sys_call(8,"gmt psxy ",Filet,map,range,tick,sbar," -Wthinnest -Glightgray -X5.0 -Y10.0 -P -K >",Filep); }

void plot_curven()
{ sys_call(6,"gmt psxy ",Filet,map,range," -Wthinner -P -O >>",Filep); }

void plot_curvenK()
{ sys_call(6,"gmt psxy ",Filet,map,range," -Wthinner -P -O -K >>",Filep); }

void plot_curvel()
{ sys_call(6,"gmt psxy ",Filet,map,range," -Wthinner -P -O >>",Filep); }

void plot_curveld()
{ sys_call(6,"gmt psxy ",Filet,map,range," -Wthin,. -P -O >>",Filep); }

void create_curve(x, dx, curve)
double x, dx, *curve;
{
	FILE *fp;
	int i;

	if ((fp=fopen(Filet,"w")) == NULL) error("Error of fopen at hst_hist!");
	for (i=0; i<NFRC; i++)
		fprintf(fp, "%.4g %.4g\n", x + dx*i, curve[i]);
	fclose(fp);
}

/* g_xmax_dx -- set up axis parameters */

void g_xmax_dx(xmax, xmin, dx, ddx)
double xmax, xmin, *dx, *ddx;
{
	double span, p=0.;

	span = xmax - xmin;
	while (span >= 10.){
		span /= 10.;
		p += 1.;
	}
	while (span < 1.){
		span *= 10.;
		p -= 1.;
	}
	if (span < 1.2){
		*dx = 0.2*pow(10.,p);   *ddx = 0.1*pow(10.,p);
	}
	else if (span < 3.0){
		*dx = 0.5*pow(10.,p);   *ddx = 0.1*pow(10.,p);
	}
	else if (span < 5.){
		*dx = 1.*pow(10.,p);   *ddx = 0.5*pow(10.,p);
	}
	else if (span < 10.){
		*dx = 2.*pow(10.,p);   *ddx = 1*pow(10.,p);
	}
	else
		error("Error in g_xmax_dx()!");
}
