/* hst_stat -- Chi2 & Kolmogorov-Smirnov test whether the data is norm & log-norm distribtion
	CAUTION: kstestn() & kstestl() will change array order of X[] by calling sort()
*/

#include <math.h>
#include "histo.h"

#define NBMIN	4
#define NBMAX	20
#define BINMIN	5
#define CNTCE	"** Due to small data number, statistical test was discontinued! **"
#define CNTC0	"Chi-square Test for normal distribution.\n"
#define CNTCH	"  nbin df    Chi2    p\n"
#define CNTC0L	"Chi-square Test for Log-normal distribution.\n"
#define CNTCP	"  %3d %3d  %7.3f  %6.4f\n"
#define CNTM	"  Mean p  Stdev of p\n  %6.4f    %6.4f\n"
#define CONTL   "  N     Mean       Stdev\n  %-4d %#- 10.4g %#- 10.4g\n"
#define CNTK0	"Kolmogorov-Smirnov Test for normal distribution.\n"
#define CNTK0L	"Kolmogorov-Smirnov Test for Log-normal distribution.\n"
#define CNTKP	"   D       p\n  %6.4f  %6.4f\n"

extern unsigned int Opt;
extern int N;
extern double *X, Mxl, Sxl, Mxg, Sxg;

static int nb, dumi, i, j, k;
static double chi2, probc, bin[20], ebin[20], D, probk, dx, ddx, dmx;

void hst_stat(mx, sx)
double mx, sx;
{

	if (N/NBMIN < BINMIN) error(CNTCE);

	if (!(Opt & NORM_BIT) && !(Opt & LOGNORM_BIT)){
		chi2test_norm(mx, sx);
		kstest_norm(mx, sx);
	}
	else if ((Opt & NORM_BIT) && !(Opt & LOGNORM_BIT)){
		chi2test_norm(mx, sx);
		kstest_norm(mx, sx);
	}
	else if (!(Opt & NORM_BIT) && (Opt & LOGNORM_BIT)){
		chi2test_lognorm(mx, sx);
		kstest_lognorm();
	}
	else {
		chi2test_norm(mx, sx);
		kstest_norm(mx, sx);
		chi2test_lognorm(mx, sx);
		kstest_lognorm();
	}
}

void chi2test_norm(mx, sx)
double mx, sx;
{
	nb = NBMIN;   i = 0;   dx = 0.;   ddx = 0.;
	fprintf(stdout, CNTC0);
	fprintf(stdout, CNTCH);
	while (nb <= NBMAX && N/nb >= BINMIN){
		k = 0;
		histn(N, X, mx, sx, nb, bin, ebin);
		for (j=0; j<nb; j++) if (bin[j] < BINMIN) k++;
		if (k > 0){
			nb++;
			continue;
		}
		chi2 = chi2test(nb, 2, bin, ebin, &probc);
		fprintf(stdout, CNTCP, nb, nb-3, chi2, probc);
		dx += probc;   ddx += probc*probc;
		nb++;   i++;
	}
	if (i == 1)
		fprintf(stdout, CNTM, dx, 0.);
	else if (i >= 2){
		dmx = dx/i;
		fprintf(stdout, CNTM, dmx, sqrt((ddx-i*dmx*dmx)/(i-1.)));
	}
}

void chi2test_lognorm(mx, sx)
double mx, sx;
{
	nb = NBMIN;   i = 0;   dx = 0.;   ddx = 0.;
	fprintf(stdout, CNTC0L);
	fprintf(stdout, CONTL, N, Mxg, Sxg);
	fprintf(stdout, CNTCH);
	while (nb <= NBMAX && N/nb >= BINMIN){
		k = 0;
		histln(N, X, Mxl, Sxl, nb, bin, ebin);
		for (j=0; j<nb; j++) if (bin[j] < BINMIN) k++;
		if (k > 0){
			nb++;
			continue;
		}
		chi2 = chi2test(nb, 2, bin, ebin, &probc);
		fprintf(stdout, CNTCP, nb, nb-3, chi2, probc);
		dx += probc;   ddx += probc*probc;
		nb++;   i++;
	}
	if (i == 1)
		fprintf(stdout, CNTM, dx, 0.);
	else if (i >= 2){
		dmx = dx/i;
		fprintf(stdout, CNTM, dmx, sqrt((ddx-i*dmx*dmx)/(i-1.)));
	}
}

void kstest_norm(mx, sx)
double mx, sx;
{
	fprintf(stdout, CNTK0);
	kstestn(N, mx, sx, X, &D, &probk);
	fprintf(stdout, CNTKP, D, probk);
}

void kstest_lognorm()
{
	fprintf(stdout, CNTK0L);
	kstestl(N, Mxl, Sxl, X, &D, &probk);
	fprintf(stdout, CNTKP, D, probk);
}
#undef NBMIN
#undef NBMAX
#undef BINMIN
#undef CNTC0
#undef CNTCH
#undef CNTC0L
#undef CNTCE
#undef CNTCP
#undef CNTK0
#undef CNTK0L
#undef CNTKP
#undef CNTM
#undef CONTL
/*
for (j=0; j<nb; j++) printf("nb,bin,ebin: %2d %5.1f %5.1f\n", nb, bin[j], ebin[j]);
*/
