/* histo2 -- Histograms of two dataset V1.0, using GMT 5, and statistical
	test  whether the two data share the same distribution.   1218,1019
    USAGE:
	histo2 file1 file2 [-Ww -Xmin/max -Ymin/max -C[N][L] -T[E] -S -H]
    Options:
	-Ww       -- bin width w
	-Xmin/max -- range of min & max for x-axis
	-Ymin/max -- range of min & max for y-axis
	(scale is common for two data, range options alter only graph appearance)
	-C[N][L]  -- curves of normal (default/N) or/and log-normal (L) dists
	-T[E]     -- plot a histogram of the totaled data with y-axis down-scaled
                     (default) or the same scale (E) as those for each data, i.e.
		     E-option extends the height of the combined histogram
	-S        -- carry out Chi-2 and K-S tests whether the two data share
		     the same distribution, also test to normal or/and log-normal
		     distributions (which dist depends on the C-option).
	-H        -- help message
    Input file:
	Two separate data files should be supplied with the next format,
	# comment lines with # (or %) at the beginning at any places
	# one datum per line at the 1st column
	    x1 [y1, z1, .. -- any extra data are ignored]
	    x2
	    .
	    .
    Output file:
	PS figure is created with the name "file1-file2.ps".
	When S option, results of statistical test are written to stdout.
    Variables:
	N1,N2,Nt,X1,X2,Xt -- numbers & arrays of two and combined data
	Xmin,Xmax -- range of data X to be plotted, common to all histograms
	    (Do not confuse with minx & maxx which are real ones)
	Ymin,Ymax,Ymaxt -- frequency range plotted for two and combined data
	dX -- bin width for X1, X2, and Xt
	Nfrq -- bin number (default NBINDFLT=22 including two edge zero bins)
	    (Do not confuse Nfrq with nbin for Chi2 test)
	Frq1,Frq2,Frqt -- frequency arrays (in default, Frq1[0]=Frq1[Nfrq-1]=0, etc.)
	    (Do not confuse Frq[] with bin[] & ebin[] for Chi2 test)
	NFRC -- number of steps to draw distribution curve (now 200)
*/

#include <stdlib.h>
#include "histo2.h"

#define HEAD "Histograms and common distribution test of %s and %s.\n\
  PS file: %s\n\
           N    Mean      Stdev       Max        Min       File\n"
#define CONT "  %s: %4d %#- 10.4g %#- 10.4g %#- 10.4g %#- 10.4g %s\n"

char File1[NF], File2[NF], Filet[NF], Filep[NF];
unsigned int Opt=0;
int N1, N2, Nt, Nfrq;
double *X1, *X2, *Xt, Xmax, Xmin, dX, Ymax, Ymin=0., Ymaxt, *Frq1, *Frq2, *Frqt, Mxl1, Sxl1, \
	Mxg1, Sxg1, Mxl2, Sxl2, Mxg2, Sxg2, Mxt, Sxt, Mxlt, Sxlt, Mxgt, Sxgt;

int main(argc, argv)
int argc;
char *argv[];
{
	double mx1, sx1, maxx1, minx1, mx2, sx2, maxx2, minx2;

	hst2_cmd(argc, argv);
	hst2_dat(&mx1, &sx1, &maxx1, &minx1, &mx2, &sx2, &maxx2, &minx2);
	fprintf(stdout, HEAD, File1, File2, Filep);
	fprintf(stdout, CONT, "Data1", N1, mx1, sx1, maxx1, minx1, File1);
	fprintf(stdout, CONT, "Data2", N2, mx2, sx2, maxx2, minx2, File2);
	if (Opt & TOTALHST_BIT)
		fprintf(stdout,CONT,"Total",Nt,Mxt,Sxt,MAX(maxx1,maxx2),MIN(minx1,minx2),"--");
	hst2_hist(mx1, sx1, mx2, sx2);
	if (Opt & STAT_BIT) hst2_stat(mx1, sx1, mx2, sx2);
	hst2_mfree();
/*	sys_call(3, "evince ", Filep, " &"); */
	sys_call(2, "rm -f ", Filet);
}
#undef HEAD
#undef CONT
