/* hst2_stat -- Chi2 & Kolmogorov-Smirnov test whether the two data share   1218
	a common distribution. Also tested to normal/log-normal distribution.
	CAUTION: kstest2, kstestn & kstestl change array order of X[] by calling sort.
*/

#include <math.h>
#include "histo2.h"

#define NBMIN	4
#define NBMAX	20
#define BINMIN	5
#define CNTCE	"** Due to small data number, statistical test was discontinued! **"
#define CNTC2	"Chi-square Test for a common distribution of two data.\n"
#define CNTC0	"Chi-square Test for normal distribution (%s).\n"
#define CNTCH	"  nbin df    Chi2    p\n"
#define CNTC0L	"Chi-square Test for Log-normal distribution (%s).\n"
#define CNTCP	"  %3d %3d  %7.3f  %6.4f\n"
#define CNTM	"  Mean p  Stdev of p\n  %6.4f    %6.4f\n"
#define CONTL   "  N     Mean       Stdev\n  %-4d %#- 10.4g %#- 10.4g\n"
#define CNTK2	"Kolmogorov-Smirnov Test for a common distribution of two data.\n"
#define CNTK0	"Kolmogorov-Smirnov Test for normal distribution (%s).\n"
#define CNTK0L	"Kolmogorov-Smirnov Test for Log-normal distribution (%s).\n"
#define CNTKP	"   D       p\n  %6.4f  %6.4f\n"

extern char File1[NF], File2[NF];
extern unsigned int Opt;
extern int N1, N2, Nt;
extern double *X1, *X2, *Xt, Mxl1, Sxl1, Mxg1, Sxg1, Mxl2, Sxl2, Mxg2, Sxg2, \
	Mxt, Sxt, Mxlt, Sxlt, Mxgt, Sxgt;

static int nb, dumi, i, j, k;
static double chi2, probc, bin[20], bin1[20], bin2[20], ebin[20], D, probk, dx, ddx, dmx;

void hst2_stat(mx1, sx1, mx2, sx2)
double mx1, sx1, mx2, sx2;
{
	if (N1/NBMIN < BINMIN || N2/NBMIN < BINMIN) error(CNTCE);
	chi2test_two();
	kstest_two();
	if ((Opt & TOTALHST_BIT) && (Opt & NORM_BIT)){
		chi2test_norm("Combined", Nt, Xt, Mxt, Sxt);
		kstest_norm("Combined", Nt, Xt, Mxt, Sxt);
	}
	if ((Opt & TOTALHST_BIT) && (Opt & LOGNORM_BIT)){
		chi2test_lognorm("Combined", Nt, Xt, Mxt, Sxt, Mxlt, Sxlt, Mxgt, Sxgt);
		kstest_lognorm("Combined", Nt, Xt, Mxlt, Sxlt);
	}
	if ((Opt & NORM_BIT) && (Opt & LOGNORM_BIT)){
		chi2test_norm(File1, N1, X1, mx1, sx1);
		kstest_norm(File1, N1, X1, mx1, sx1);
		chi2test_lognorm(File1, N1, X1, mx1, sx1, Mxl1, Sxl1, Mxg1, Sxg1);
		kstest_lognorm(File1, N1, X1, Mxl1, Sxl1);
		chi2test_norm(File2, N2, X2, mx2, sx2);
		kstest_norm(File2, N2, X2, mx2, sx2);
		chi2test_lognorm(File2, N2, X2, mx2, sx2, Mxl2, Sxl2, Mxg2, Sxg2);
		kstest_lognorm(File2, N2, X2, Mxl2, Sxl2);
	}
	else if (Opt & NORM_BIT){
		chi2test_norm(File1, N1, X1, mx1, sx1);
		kstest_norm(File1, N1, X1, mx1, sx1);
		chi2test_norm(File2, N2, X2, mx2, sx2);
		kstest_norm(File2, N2, X2, mx2, sx2);
	}
	else if (Opt & LOGNORM_BIT){
		chi2test_lognorm(File1, N1, X1, mx1, sx1, Mxl1, Sxl1, Mxg1, Sxg1);
		kstest_lognorm(File1, N1, X1, Mxl1, Sxl1);
		chi2test_lognorm(File2, N2, X2, mx2, sx2, Mxl2, Sxl2, Mxg2, Sxg2);
		kstest_lognorm(File2, N2, X2, Mxl2, Sxl2);
	}
}

/* chi2test_two -- Chi-2 test for a common distribution of two data sets */

void chi2test_two()
{
	nb = NBMIN;   i = 0;   dx = 0.;   ddx = 0.;
	fprintf(stdout, CNTC2);
	fprintf(stdout, CNTCH);
	while (nb <= NBMAX && N1/nb >= BINMIN && N2/nb >= BINMIN){
		k = 0;
		hist2c(N1, N2, X1, X2, nb, bin1, bin2);
		for (j=0; j<nb; j++) if (bin1[j]<BINMIN || bin2[j]<BINMIN) k++;
		if (k > 0){
			nb++;
			continue;
		}
		chi2 = chi2test2(nb, 1, bin1, bin2, &probc);
		fprintf(stdout, CNTCP, nb, nb-1, chi2, probc);
		dx += probc;   ddx += probc*probc;
		nb++;   i++;
	}
	if (i == 1)
		fprintf(stdout, CNTM, dx, 0.);
	else if (i >= 2){
		dmx = dx/i;
		fprintf(stdout, CNTM, dmx, sqrt((ddx-i*dmx*dmx)/(i-1.)));
	}
}

/* kstest_two -- Kolmogorov-Smirnov test to a common distribution */

void kstest_two()
{
	fprintf(stdout, CNTK2);
	kstest2(N1, X1, N2, X2, &D, &probk);
	fprintf(stdout, CNTKP, D, probk);
}

/* chi2test_norm -- Chi-2 test to a normal distribution */

void chi2test_norm(str, n, x, mx, sx)
char *str;
int n;
double *x, mx, sx;
{
	nb = NBMIN;   i = 0;   dx = 0.;   ddx = 0.;
	fprintf(stdout, CNTC0, str);
	fprintf(stdout, CNTCH);
	while (nb <= NBMAX && n/nb >= BINMIN){
		k = 0;
		histn(n, x, mx, sx, nb, bin, ebin);
		for (j=0; j<nb; j++) if (bin[j] < BINMIN) k++;
		if (k > 0){
			nb++;
			continue;
		}
		chi2 = chi2test(nb, 2, bin, ebin, &probc);
		fprintf(stdout, CNTCP, nb, nb-3, chi2, probc);
		dx += probc;   ddx += probc*probc;
		nb++;   i++;
	}
	if (i == 1)
		fprintf(stdout, CNTM, dx, 0.);
	else if (i >= 2){
		dmx = dx/i;
		fprintf(stdout, CNTM, dmx, sqrt((ddx-i*dmx*dmx)/(i-1.)));
	}
}

/* chi2test_lognorm -- Chi-2 test to a log-normal distribution */

void chi2test_lognorm(str, n, x, mx, sx, mxl, sxl, mxg, sxg)
char *str;
int n;
double *x, mx, sx, mxl, sxl, mxg, sxg;
{
	nb = NBMIN;   i = 0;   dx = 0.;   ddx = 0.;
	fprintf(stdout, CNTC0L, str);
	fprintf(stdout, CONTL, n, mxg, sxg);
	fprintf(stdout, CNTCH);
	while (nb <= NBMAX && n/nb >= BINMIN){
		k = 0;
		histln(n, x, mxl, sxl, nb, bin, ebin);
		for (j=0; j<nb; j++) if (bin[j] < BINMIN) k++;
		if (k > 0){
			nb++;
			continue;
		}
		chi2 = chi2test(nb, 2, bin, ebin, &probc);
		fprintf(stdout, CNTCP, nb, nb-3, chi2, probc);
		dx += probc;   ddx += probc*probc;
		nb++;   i++;
	}
	if (i == 1)
		fprintf(stdout, CNTM, dx, 0.);
	else if (i >= 2){
		dmx = dx/i;
		fprintf(stdout, CNTM, dmx, sqrt((ddx-i*dmx*dmx)/(i-1.)));
	}
}

/* kstest_norm -- Kolmogorov-Smirnov test to a normal distribution */

void kstest_norm(str, n, x, mx, sx)
char *str;
int n;
double *x, mx, sx;
{
	fprintf(stdout, CNTK0, str);
	kstestn(n, mx, sx, x, &D, &probk);
	fprintf(stdout, CNTKP, D, probk);
}

/* kstest_lognorm -- Kolmogorov-Smirnov test to a log-normal distribution */

void kstest_lognorm(str, n, x, mxl, sxl)
char *str;
int n;
double *x, mxl, sxl;
{
	fprintf(stdout, CNTK0L, str);
	kstestl(n, mxl, sxl, x, &D, &probk);
	fprintf(stdout, CNTKP, D, probk);
}
#undef NBMIN
#undef NBMAX
#undef BINMIN
#undef CNTCE
#undef CNTC2
#undef CNTC0
#undef CNTCH
#undef CNTC0L
#undef CNTCP
#undef CNTM
#undef CONTL
#undef CNTK2
#undef CNTK0
#undef CNTK0L
#undef CNTKP
/*
for (j=0; j<nb; j++) printf("nb,bin1,bin2: %2d %5.1f %5.1f\n", nb, bin1[j], bin2[j]);
for (j=0; j<nb; j++) printf("nb,bin,ebin: %2d %5.1f %5.1f\n", nb, bin[j], ebin[j]);
*/
