/* pcr_fig -- plot the fitted circle with the data using GMT5 (classic mode of GMT 6)
		array size of tick increased.   031921
*/

#include <string.h>
#include <math.h>
#include "pcirc.h"

#define MAPT " -Jx1c -R0/12.5/0/3.0 -Y-3.0"
#define TEXT "2.50 1.00 10p,Helvetica-Bold,black CM N=%d\n\
6.00 1.00 10p,Helvetica-Bold,black CM X@-0@-=%-.4g\n\
9.50 1.00 10p,Helvetica-Bold,black CM Y@-0@-=%-.4g\n\
2.50 0.50 10p,Helvetica-Bold,black CM R=%-.4g\n\
6.00 0.50 10p,Helvetica-Bold,black CM k=%-.4g\n\
9.50 0.50 10p,Helvetica-Bold,black CM SSE=%-.4g"
#define OPTF " -F+f+j"

extern char File[NF], Filet[NF], Filep[NF];
extern unsigned int Opt;
extern int N;
extern double *X, *Y, *Th, X0, Y0, R, K, Sse, Xmax, Xmin, Ymax, Ymin;

void pcr_fig()
{
	FILE *fp;
	char map[50], range[100], tick[300];
	int i, i1=0, i2=360;
	double maxx, minx, maxy, miny, rangep, spanx, spany, xc[361], yc[361],\
		maxth, minth, dxg, ddxg, dyg, ddyg, dum;

	/* set up circle points */

	if (Opt & ARC_BIT){
		for (i=0; i<N; i++){
			Th[i] = RAD*atan2(Y[i]-Y0, X[i]-X0);
			checklon(&Th[i], 0);
		}
		maxmin(N, Th, &maxth, &minth);
		i1 = (int)minth;  i2 = (int)(maxth + 0.5);
	}
	for (i=i1; i<=i2; i++){
		dum = i;
		xc[i] = X0 + R*cos(dum/RAD);  yc[i] = Y0 + R*sin(dum/RAD);
	}

	/* X & Y ranges, the same scale if no range options */

	maxmin(N, X, &maxx, &minx);  maxmin(N, Y, &maxy, &miny);
	if (!(Opt & XRANGE_BIT)){
		Xmax = (maxx > X0+R) ? maxx : X0+R;
		Xmin = (minx < X0-R) ? minx : X0-R;
		rangep = RANGEFACT*(Xmax - Xmin);
		Xmax = Xmax + rangep;  Xmin = Xmin - rangep;
	}
	if (!(Opt & YRANGE_BIT)){
		Ymax = (maxy > Y0+R) ? maxy : Y0+R;
		Ymin = (miny < Y0-R) ? miny : Y0-R;
		rangep = RANGEFACT*(Ymax - Ymin);
		Ymax = Ymax + rangep;  Ymin = Ymin - rangep;
	}
	if (!(Opt & XRANGE_BIT) && !(Opt & YRANGE_BIT)){
		spanx = Xmax - Xmin;  spany = Ymax - Ymin;
		if (spanx >= spany) Ymax = Ymin + spanx;
		else Xmax = Xmin + spany;
	}

	/* set up axis parameters & initialize GMT */

	sprintf(map, " -JX%.2fc/%.2fc", MAPWIDTH, MAPHEIGHT);
	g_xmax_dx(Xmax, Xmin, &dxg, &ddxg);
	g_xmax_dx(Ymax, Ymin, &dyg, &ddyg);
	sprintf(range, " -R%.4g/%.4g/%.4g/%.4g", Xmin, Xmax, Ymin, Ymax);
	sprintf(tick," -BWeSn+t\"Circle Fitting of %s\" -Bxa%.4gf%.4g+lX -Bya%.4gf%.4g+LY",File,dxg,ddxg,dyg,ddyg);
	sys_call(2, "gmt gmtset", GSET);

	/* plot data & the circle */

	if ((fp=fopen(Filet,"w")) == NULL) error("Error of fopen at pcr_fig!");
	for (i=0; i<N; i++)
		fprintf(fp, "%.4g %.4g\n", X[i], Y[i]);
	fclose(fp);
	sys_call(7, "gmt psxy ",Filet,map,range,tick," -Sc0.15c -G0/0/205 -Wthin,0/0/205 -X5.0 -Y10.0 -P -K >",Filep);
	if (Opt & CENTER_BIT){
		if ((fp=fopen(Filet,"w")) == NULL) error("Error of fopen at pcr_fig!");
		fprintf(fp, "%.4g %.4g\n", X0, Y0);
		fclose(fp);
		sys_call(7, "gmt psxy ",Filet,map,range,tick," -Sc0.20c -Gred -Wthin,red -P -O -K >>",Filep);
	}
	if ((fp=fopen(Filet,"w")) == NULL) error("Error of fopen at pcr_fig!");
	for (i=i1; i<=i2; i++)
		fprintf(fp, "%.4g %.4g\n", xc[i], yc[i]);
	fclose(fp);
	sys_call(6, "gmt psxy ",Filet,map,range," -Wthin,red -P -O -K >>",Filep);

	/* plot texts */

	if ((fp=fopen(Filet,"w")) == NULL) error("Error of fopen at pcr_fig!");
	fprintf(fp, TEXT, N, X0, Y0, R, K, Sse);
	fclose(fp);
	sys_call(6, "gmt pstext ", Filet, MAPT, OPTF, " -P -O >>", Filep);
}

/* g_xmax_dx -- set up axis parameters */

void g_xmax_dx(xmax, xmin, dx, ddx)
double xmax, xmin, *dx, *ddx;
{
	double span, p=0.;

	span = xmax - xmin;
	while (span >= 10.){
		span /= 10.;
		p += 1.;
	}
	while (span < 1.){
		span *= 10.;
		p -= 1.;
	}
	if (span < 1.2){
		*dx = 0.2*pow(10.,p);   *ddx = 0.1*pow(10.,p);
	}
	else if (span < 3.0){
		*dx = 0.5*pow(10.,p);   *ddx = 0.1*pow(10.,p);
	}
	else if (span < 5.){
		*dx = 1.*pow(10.,p);   *ddx = 0.5*pow(10.,p);
	}
	else if (span < 10.){
		*dx = 2.*pow(10.,p);   *ddx = 1*pow(10.,p);
	}
	else
		error("Error in g_xmax_dx()!");
}
