/* pdmg_cmd -- command interpretation for pdemag */

#include <ctype.h>
#include <string.h>
#include "pdemag.h"

#define USAGE   "Usage: pdemag [-H] file"
#define HELP   "HELP MESSAGE OF PDEMAG\n\
  Analysis of Progressive Demagnetization V.1.4, H. Tanaka, 2019\n\
  Plots use GMT 6.3.0 classic mode (GMT 5.4.3) of Wessel et al. 2019\n\n\
Usage:\n\
  pdemag [-H] file\n\n\
Options:\n\
  H -- show this help message\n\n\
Commands during analysis:\n\
  \'+\'-next, \'-\'-back, \'0\'-first, \'9\'-last, \'C\'ore, in-\'S\'itu, \'B\'ed-corrected,\n\
  \'L\'ine-pca, \'G\'tc-pca, critical-\'M\'ad, o\'R\'igin-on/off, \'A\'nchor-on/off, au\'T\'o-on/off, \n\
  \'D\'elete-results, \'W\'rite-results, \'P\'sfile, e\'N\'large-scale, ann\'O\'tate-on/on2/off,\n\
  \'E\'qual-area, \'Z\'ijderveld, \'H\'elp, \'Q\'uit\n\n\
Input file:\n\
  Data file should contain one or more sample-level data from a single site.\n\
  Any lines with 1st letter \'#\'/\'%\' at any position are comments.\n\
  Orientation info at $ line can be omitted, in which case zero is assigned.\n\n\
  $ sample1 [azim1 plun1 [strik1 dip1]] -- every sample data begins with $.\n\
     0 M Ic Dc [chi]                    -- step data in core coordinates.\n\
     5 M Ic Dc [chi]                    -- chi data may be included at 5th column.\n\
            .\n\
            .\n\
  $ sample2 [azim2 plun2 [stik2 dip2]]\n\
    20 M Ic Dc [chi]\n\
   100 M Ic Dc [chi]\n\
            .\n\
            .\n\n\
Output file:\n\
  1. W command writes PCA-line & PCA-GTC results to the user specified file,\n\
  which may contain several site data. Data is appended to an existing file.\n\
  The output file will be a data file to be read by \"pdirec\".\n\n\
  $ input_file1                                       -- site data begins with $\n\
    sample1 Ic Dc Is Ds It Dt MAD THD  20-80          -- PCA-line result\n\
    sample2 Ic Dc Is Ds It Dt MAD  A  150-500         -- anchored result\n\
             .\n\
             .\n\
    sample5 Ipc Dpc Ia1c Da1c Ia2c Da2c MAD P 20-600  -- three lines for\n\
            Ips Dps Ia1s Da1s Ia2s Da2s               -- PCA-GTC result\n\
            Ipt Dpt Ia1t Da1t Ia2t Da2t\n\
             .\n\
             .\n\
  $ input_file2\n\
    sample1 Ic Dc Is Ds It Dt MAD THD 5-100\n\
             .\n\
             .\n\n\
  2. P command creates a PS file with user specified name.\n\
  3. \"PDEMAG.PS\" is re-written every time a new sample is read.\n\
(Q to quit)"

FILE *pdmg_cmd(argc, argv, file)
int argc;
char *argv[], *file;
{
	FILE *fp;
	char *s;
	int flagf = 0;

	if (argc < 2) error(USAGE);
	while (--argc > 0){
		if (*argv[argc] == '-'){
			s = argv[argc] + 1;
			switch (toupper(*s)){
			case 'H':
				showless(HELP);
				break;
			default:
				error("Unknown option!");
				break;
			}
		}
		else {
			if (flagf != 0) error("Assign only one file!");
			strcpy(file, argv[argc]);
			if ((fp = fopen(file, "r")) == NULL) error("Error of open file!");
			flagf = 1;
		}
	}
	if (flagf != 0) return(fp);
	else error("Error of open file!");
}
#undef USAGE
#undef HELP
