/* pdmg_dat -- read data from a file and stores as a series in Magd[], Incd[], ...
	Internal variables:
		i -- counts number of samples
		j -- counts number of data lines in series through samples
		k -- counter of steps for each sample
		l -- denotes core, in situ, tilted frames by 0, 1, 2.
		nstepmax -- used as dimension of all step related variables
	Return:
		number of data lines in the original data file
*/

#include <string.h>
#include <stdlib.h>
#include "pdemag.h"

extern char **Sampl, **Stepd;
extern int Nd, *Idsmpl, *Ida;
extern double *Azim, *Plun, *Strik, *Dip, *Magd, *Incd, *Decd, *Chid, *Step, *Mag, *Chi, \
	**Is, **Ds, **Xs, **Ys, **Zs;
extern double **Ieq, **Deq, **Ieqr, **Xzj, **Yzj, **Zzj, *Yjn, *Ych;

int pdmg_dat(fp)
FILE *fp;
{

	char s1[50], s2[50], s3[30], s4[30], s5[30], s6[30];
	int n, i = 0, j = 0, k = 0, nstepmax = 0, l = 0, nline;

	while ((n = get_s(fp, 6, s1, s2, s3, s4, s5, s6)) != EOF){
		if (n == 0 || s1[0] == '#' || s1[0] == '%') continue;
		if (strcmp("$", s1) == 0) i++;
		else j++;
	}
	Nd = i;   nline = j;
	pdmg_mem1(Nd, nline);
	i = 0;   j = 0;   rewind(fp);
	while ((n = get_s(fp, 6, s1, s2, s3, s4, s5, s6)) != EOF){
		if (n == 0 || s1[0] == '#' || s1[0] == '%') continue;
		if (strcmp("$", s1) == 0){
			if (n == 1) error("Error in $ line!");
			Idsmpl[i] = j;
			strcpy(Sampl[i], s2);
			if (n == 2){
				Azim[i] = Plun[i] = Strik[i] = Dip[i] = 0.;
			}
			else if (n == 3){
				Azim[i] = atof(s3);
				Plun[i] = Strik[i] = Dip[i] = 0.;
			}
			else if (n == 4){
				Azim[i] = atof(s3);  Plun[i] = atof(s4);
				Strik[i] = Dip[i] = 0.;
			}
			else if (n == 5){
				Azim[i] = atof(s3);  Plun[i] = atof(s4);
				Strik[i] = atof(s5);  Dip[i] = 0.;
			}
			else {
				Azim[i] = atof(s3);  Plun[i] = atof(s4);
				Strik[i] = atof(s5);  Dip[i] = atof(s6);
			}
			count_step(&k, &nstepmax);
			i++;
		}
		else {
			strcpy(Stepd[j], s1);
			Magd[j] = atof(s2);   Incd[j] = atof(s3);   Decd[j] = atof(s4);
			if (n == 4) Chid[j] = 0.;
			else Chid[j] = atof(s5);
			k++;   j++;
		}
	}
	if (i == 0) error("Error of data file!");
	Idsmpl[i] = j;
	count_step(&k, &nstepmax);
	pdmg_mem2(nstepmax);
	return(nline);
}

/* count_step */

void count_step(k, nstepmax)
int *k, *nstepmax;
{
	int nstep;

	nstep = *k;
	if (nstep > *nstepmax) *nstepmax = nstep;
	*k = 0;
}

/* pdmg_mem1 -- memory allocation for Magd[], Incd[], Decd[]
	nd -- number of data or sample, nl -- number of data line in original file
*/

void pdmg_mem1(nd, nl)
int nd, nl;
{
	Stepd = mallocms(nl, WSTEP);
	Magd = mallocvd(nl);   Incd = mallocvd(nl);   Decd = mallocvd(nl);
	Chid = mallocvd(nl);

	Sampl = mallocms(nd, WSAMP);
	Idsmpl = mallocvi(nd);
	Azim = mallocvd(nd);   Plun = mallocvd(nd);
	Strik = mallocvd(nd);  Dip  = mallocvd(nd);
}

/* pdmg_mem2 -- memory allocation for step related variables for both remanence vector
		and GMT plotting
*/

void pdmg_mem2(ns)
int ns;
{
	Step = mallocvd(ns); Mag = mallocvd(ns);  Chi = mallocvd(ns);

	Is = mallocmd(3, ns);   Ds = mallocmd(3, ns);
	Xs = mallocmd(3, ns);   Ys = mallocmd(3, ns);   Zs = mallocmd(3, ns);

	Ida = mallocvi(ns);
	Ieq = mallocmd(3, ns);   Deq = mallocmd(3, ns);   Ieqr = mallocmd(3, ns);
	Xzj = mallocmd(3, ns);   Yzj = mallocmd(3, ns);   Zzj = mallocmd(3, ns);
	Yjn = mallocvd(ns);  Ych = mallocvd(ns);
}

/* free_all -- free memory for all external array variables */

void free_all(nline)
int nline;
{
	freems(nline, Stepd);   freems(Nd, Sampl);

	freevi(Idsmpl);  freevi(Ida);

	freevd(Azim);  freevd(Plun);  freevd(Strik);  freevd(Dip);
	freevd(Magd);  freevd(Incd);  freevd(Decd);  freevd(Chid);
	freevd(Step);  freevd(Mag);   freevd(Chi);

	freemd(3, Is);   freemd(3, Ds);
	freemd(3, Xs);   freemd(3, Ys);   freemd(3, Zs);

	freemd(3, Ieq);   freemd(3, Deq);   freemd(3, Ieqr);
	freemd(3, Xzj);   freemd(3, Yzj);   freemd(3, Zzj);
	freevd(Yjn);  freevd(Ych);
}
