#include <string.h>
#include "pdemag.h"

#define MAPJN   " -JX5.0/5.0"
#define MAPJNT  " -Jx1 -R0/5.0/0/5.0"
#define TEXTJN  "4.90 4.90 12p,Helvetica-Bold,black RT Jn@-0@- = %s\n\
4.90 4.50 12p,Helvetica,black RT MDF%s\n\
4.90 4.10 12p,Helvetica,black RT QDF%s\n"
#define TEXTCH  "4.90 4.90 12p,Helvetica-Bold,black RT @~c@~@-20@- = %s\n"
#define CONTRJN " -R0/%.0f/0/%.1f"
#define CONTTJN " -Bf%.0fa%.0f:\"Step (mT/\\260C)\":/f%.1fa%.1f:\"Jn/Jn@-0@-\":WeSn"
#define CONTRCH " -R0/%.0f/%.1f/%.1f"
#define CONTTCH " -Bf%.0fa%.0f:\"Step (\\260C)\":/f%.1fa%.1f:\"@~c@~/@~c@~@-20@-\":WeSn"
#define CONT "%.3f %.3f\n"

extern char *Circs[], Sdum[LSIZ_COM];
extern int Nstep, I1, I2;
extern double *Mag, *Step, *Yjn, Yjnmax, dYjna, dYjnt, Xjnmax, dXjna, dXjnt, \
		*Chi, *Ych, Ychmax, Ychmin, dYcha, dYcht;

static char Range[100], Tick[100], Ylabel[50], Mdflbl[20], Qdflbl[20];

/* pdmg_jn -- plot a Jn-Step plot 
	V.2: print out of half & 1/4 destructive field is introduced   052506
*/

void pdmg_jn(xy)
char *xy;
{
	/* set up axis & tick parameters */

	sprintf(Range, CONTRJN, Xjnmax, Yjnmax);
	sprintf(Tick, CONTTJN, dXjnt, dXjna, dYjnt, dYjna);
	sys_call(7, "gmt psbasemap ", MAPJN, Tick, Range, xy, " -P -K >", FILEP);

	/* plot all the Jn-step data */

	wrt_tmp2(0, Nstep-1, CONT, Step, Yjn);
	sys_call(7, "gmt psxy ", FILET, MAPJN, Range, PEN3S, " -P -O -K >>", FILEP);
	sys_call(7, "gmt psxy ", FILET, MAPJN, Range, Circs[0], " -P -O -K >>", FILEP);

	/* find MDF & M25 and plot texts */

	sprintf(Ylabel, "%.2e", Mag[0]);
	find_mdf(0.5, Mdflbl);
	find_mdf(0.25, Qdflbl);
	sprintf(Sdum, TEXTJN, Ylabel, Mdflbl, Qdflbl);
	wrt_tmp();
	sys_call(6, "gmt pstext ", FILET, MAPJNT, OPTF, " -N -P -O -K >>", FILEP);
}

/* pdmg_chi -- plot a Chi-Step plot */

void pdmg_chi(xy)
char *xy;
{
	/* set up axis & tick parameters */

	sprintf(Range, CONTRCH, Xjnmax, Ychmin, Ychmax);
	sprintf(Tick, CONTTCH, dXjnt, dXjna, dYcht, dYcha);
	sys_call(7, "gmt psbasemap ", MAPJN, Tick, Range, xy, " -P -O -K >>", FILEP);

	/* plot all the Jn-step data */

	wrt_tmp2(0, Nstep-1, CONT, Step, Ych);
	sys_call(7, "gmt psxy ", FILET, MAPJN, Range, PEN3S, " -P -O -K >>", FILEP);
	sys_call(7, "gmt psxy ", FILET, MAPJN, Range, Circs[0], " -P -O -K >>", FILEP);

	/* plot texts */

	sprintf(Ylabel, "%.2e", Chi[0]);
	sprintf(Sdum, TEXTCH, Ylabel);
	wrt_tmp();
	sys_call(6, "gmt pstext ", FILET, MAPJNT, OPTF, " -N -P -O -K >>", FILEP);
}

/* find_mdf -- simple interpolation by y = ax + b, but when last point is close to 
		yc (<=1.2*yc), extrapolation is also applied 
*/

void find_mdf(yc, label)
double yc;
char *label;
{
	int i, c = 0;
	double mdf, f = 1.2, x1, y1, x2, y2, a, b;

	if (Nstep <= 1){
		strcpy(label, ": NA");
		return;
	}
	for (i = 1; i < Nstep; i++){
		x1 = Step[i-1]; y1 = Yjn[i-1];
		x2 = Step[i];   y2 = Yjn[i];
		if (y2 <= yc){
			c = 1;
			break;
		}
	}
	if (y2 <= f*yc) c = 1;
	if (c == 1){
		a = (y2 - y1)/(x2 - x1);
		b = (x2*y1 -x1*y2)/(x2 - x1);
		mdf = (yc - b)/a;
		sprintf(label, " = %.0f", mdf);
	}
	else{
		mdf = Step[Nstep-1];
		sprintf(label, ": GT %.0f", mdf);
	}
}
#undef MAPJN
#undef MAPJNT
#undef TEXTJN
#undef TEXTCH
#undef CONTRJN
#undef CONTTJN
#undef CONTRCH
#undef CONTTCH
#undef CONT
