/* pdrc_cmd -- interpretation of argc & argv when pdirec was made to run
	compatibility to GMT 6.3 is anounced   120624
*/

#include <ctype.h>
#include <string.h>
#include "pdirec.h"

#define USAGE   "Usage: pdirec [-H] file"
#define HELP   "HELP MESSAGE OF PDIREC\n\
  Analysis of Paleodirections V.1.2, H. Tanaka (2024,2018)\n\
  Adapted to GMT 6.3.0 classic mode (GMT 5.4.3) by Wessel et al. (2019)\n\n\
Usage:\n\
  pdirec [-H] file\n\n\
Options:\n\
  H -- show this help message\n\n\
Commands during analysis:\n\
  \'+\'-next, \'-\'-back, \'0\'-first, \'9\'-last, \'8\'-10next, \'1\'-10back,\n\
  \'C\'ore, in-\'S\'itu, \'B\'ed-corrected, so\'U\'thern-hemi-on/off,\n\
  \'M\'ad-circle-on/off, \'D\'elete-one, \'R\'estore-one, iteration-\'I\'nitial,\n\
  \'W\'rite-results, \'P\'sfile, \'H\'elp, \'Q\'uit\n\n\
Input file:\n\
  Data file should contain one or more site-level data, each of which consists\n\
  of one or more sample directions. The input file should be the output file\n\
  from \"pdemag\", but (lat.,long) of site location should be included manually.\n\
  Any text file with the following format is also treated.\n\
  Any lines with 1st letter \"#\"/\"%\" at any position are comments.\n\n\
  $ site1 slat slon                             -- site data begins with $\n\
    sample1 Ic Dc Is Ds It Dt [Mad] [comments]\n\
    sample2 Ic Dc Is Ds It Dt [Mad] [comments]\n\
             .\n\
             .\n\
    sample5 Ipc Dpc Ia1 Da1 Ia2 Da2 MAD P [comments]  -- GTC data should\n\
            Ips Dps Ia1 Da1 Ia2 Da2     P [comments]  -- occupy succesive\n\
            Ipt Dpt Ia1 Da1 Ia2 Da2     P [comments]  -- three lines\n\
             .\n\
             .\n\
  $ site2 [slat] [slon]                 -- previous site succeeds when omitted\n\
    sample1 Ic Dc Is Ds It Dt [Mad] [comments]\n\
    sample2 Ic Dc Is Ds It Dt [Mad] [comments]\n\
             .\n\
             .\n\n\
Output files:\n\
  1. W command writes the site means & statistics to user specified file,\n\
  in which other results may be appended from different input files.\n\
  The output file will be a data file to be read by \"pstat\" or \"psvar\".\n\n\
     $ input_file1\n\
       site1 Im Dm a95 N R K plat plot dp dm asd asdv slat slon N0 Ng Coordinate\n\
       site2 Im Dm a95 N R K plat plot dp dm asd asdv slat slon N0 Ng Coordinate\n\
       (N, N0, Ng are no.s of statistics, total, & GTC data)\n\
                 .\n\
                 .\n\
     $ input_file2\n\
       site1 Im Dm a95 N R K plat plot dp dm asd asdv slat slon N0 Ng Coordinate\n\
                 .\n\
                 .\n\n\
  2. P command creates a PS file with user specified name.\n\
  3. \"PDIREC.PS\" is re-written every time a new sample is read.\n\
(Q to quit)"

extern unsigned int Opt;

FILE *pdrc_cmd(argc, argv, file)
int argc;
char *argv[], *file;
{
	FILE *fp;
	char *s;
	int flagf = 0;

	if (argc < 2) error(USAGE);
	while (--argc > 0){
		if (*argv[argc] == '-'){
			s = argv[argc] + 1;
			switch (toupper(*s)){
			case 'H':
				showless(HELP);
				break;
			default:
				error("Unknown option!");
				break;
			}
		}
		else {
			if (flagf != 0) error("Assign only one file!");
			strcpy(file, argv[argc]);
			if ((fp = fopen(file, "r")) == NULL) error("Error of open file!");
			flagf = 1;
		}
	}
	if (flagf != 0) return(fp);
	else error("Error of open file!");
}
#undef USAGE
#undef HELP
