#include <string.h>
#include "pdirec.h"

#define CONTW0 "$ %s\n"
#define CONTW  "%8s %5.1f %5.1f %5.1f %2d %6.3f %6.1f %5.1f %5.1f %5.1f %5.1f %5.1f %5.1f %5.1f %5.1f %2d %2d %s\n"

extern FILE *Fpw;
extern char Filed[WFILE], Filew[WFILE], **Site, *Frame_lbl[];
extern int Nsite, Ns, Nsg, Nst, Nstg;
extern unsigned int Opt, Frm;
extern double *Slt, *Sln, Im, Dm, R, K, A95, Asd, Vltm, Vlnm, dP, dM, Asdv;

static int c;

/* pdrc_comd -- returns a first letter of the hitted command */

int pdrc_comd(rem)
char *rem;
{
	fprintf(stdout, "%s ", rem);
	c = get_ch();
	if (c == ZERO) return(-1);
	else if (c == 'q' || c == 'Q') return ZERO;
	else return(c);
}

/* do_init -- called when the site or the coordinate frame is changed */

void do_init(n)
int n;
{
	fprintf(stdout, "%s (%s)\n", Site[n], Frame_lbl[Frm]);
	pdrc_sitedat(n);
	pdrc_plot(n);
}

/* go_next */

void go_next(n)
int *n;
{
	if (*n >= Nsite -1){
		fprintf(stdout, "End of file!\n");
		return;
	}
	*n += 1;
	do_init(*n);
}

/* go_back -- go back to the previous sample */

void go_back(n)
int *n;
{
	if (*n <= 0){
		fprintf(stdout, "Beginning of file!\n");
		return;
	}
	*n -= 1;
	do_init(*n);
}

/* go_first */

void go_first(n)
int *n;
{
	if (*n <= 0){
		fprintf(stdout, "Beginning of file!\n");
		return;
	}
	*n = 0;
	do_init(*n);
}

/* go_last */

void go_last(n)
int *n;
{
	if (*n >= Nsite - 1){
		fprintf(stdout, "End of file!\n");
		return;
	}
	*n = Nsite - 1;
	do_init(*n);
}

/* go_plus10 -- go forward by 10 samples */

void go_plus10(n)
int *n;
{
	if (*n >= Nsite - 1){
		fprintf(stdout, "End of file!\n");
		return;
	}
	*n += 10;
	if (*n > Nsite - 1)
		*n = Nsite - 1;
	do_init(*n);
}

/* go_minus10 -- go backward by 10 samples */

void go_minus10(n)
int *n;
{
	if (*n <= 0){
		fprintf(stdout, "Beginning of file!\n");
		return;
	}
	*n -= 10;
	if (*n < 0)
		*n = 0;
	do_init(*n);
}

/* core_orient -- change orientation to core frame */

void core_orient(n)
int n;
{
	if (Frm == CORE) return;
	Frm = CORE;
	do_init(n);
/*	pdrc_plot(n); */
}

/* situ_orient -- change orientation to in situ frame */

void situ_orient(n)
int n;
{
	if (Frm == SITU) return;
	Frm = SITU;
	do_init(n);
/*	pdrc_plot(n); */
}

/* bed_orient -- change orientation to bedding corrected frame */

void bed_orient(n)
int n;
{
	if (Frm == TILT) return;
	Frm = TILT;
	do_init(n);
/*	pdrc_plot(n); */
}

/* change_south -- change over between northern & southern hemispheres for VGP plot */

void change_south(n)
int n;
{
	if ( !(Opt & S_HEMI_BIT) )
		Opt |= S_HEMI_BIT;
	else
		Opt &= ~S_HEMI_BIT;
	pdrc_plot(n);
}

/* put_mad -- on & off the circle of MAD to each field direction */

void put_mad(n)
int n;
{
	if ( !(Opt & ENBL_MAD_BIT) )
		Opt |= ENBL_MAD_BIT;
	else
		Opt &= ~ENBL_MAD_BIT;
	pdrc_plot(n);
}

/* delete_one -- delete one farthest result from the Fisher statistics */

void delete_one(n)
int n;
{
	if (Nsg == 0){
		if (Nst <= 2) return;
		pdrc_rejectf(n);
		pdrc_statf(n);
	}
	else {
		if (Nst==0 && Nstg<=3) return;
		if (Nst==1 && Nstg==1) return;
		pdrc_rejectg(n);
		pdrc_statg(n, 0., 0.);
	}
	pdrc_plot(n);
}

/* restore_one -- restore the nearest deleted data to the Fisher statistics */

void restor_one(n)
int n;
{
	if (Nsg == 0){
		if (Nst >= Ns) return;
		pdrc_restorf(n);
		pdrc_statf(n);
	}
	else {
		if (Nst+Nstg >= Ns+Nsg) return;
		pdrc_restorg(n);
		pdrc_statg(n, 0., 0.);
	}
	pdrc_plot(n);
}

/* it_initial -- set (inci,deci) in iteration for all gtc data with no lines */

void it_initial(n)
int n;
{
	double inci, deci;

	if ( !(Opt & IT_INITIAL_BIT) )
		fprintf(stdout, "Not applicable for the data.\n");
	else {
		while ((c=inpd("Initial inc, dec?",2,&inci,&deci)) != ZERO){
			if (c == -1) continue;
			else break;
		}
		if (c == ZERO) return;
		pdrc_statg(n, inci, deci);
		pdrc_plot(n);
	}
}

/* write_results */

void write_results(n)
int n;
{
	if (!(Opt & WRT_FILE_BIT)){
		while ((c=inpstr("Output file?",Filew,WFILE)) != ZERO){
			if (strcmp(Filew, Filed) == 0){
				fprintf(stdout, "Name other than now input file \"%s\"!\n", Filed);
				continue;
			}
			if ((Fpw=fopen(Filew,"r")) != NULL){
				fclose(Fpw);
				if ((Fpw=fopen(Filew,"a")) != NULL){
					fprintf(stdout, "An old file %s is opened to append.\n", Filew);
					if (yorn("OK? (Y or Rtn/N)",1) != 1){
						fclose(Fpw);
						continue;
					}
					break;
				}
				else
					error("Error of open an old file!");
			}
			else {
				if ((Fpw=fopen(Filew,"w")) != NULL){
					fprintf(stdout, "A new file %s is opened.\n", Filew);
					break;
				}
				else
					error("Error of open a new file!");
			}
		}
		if (c == ZERO) return;
		fprintf(Fpw, CONTW0, Filed);
		Opt |= WRT_FILE_BIT;
	}
	else {
		if ((Fpw=fopen(Filew,"a")) == NULL)
			error("Error of an existing file!");
	}
	if (Ns==0 && Nsg==1)
		fprintf(stdout, "Single GTC result was discarded!\n");
	else {
		fprintf(Fpw, CONTW, Site[n], Im, Dm, A95, Nst+Nstg, R, K, Vltm, Vlnm, \
			dP, dM, Asd, Asdv, Slt[n], Sln[n], Ns+Nsg, Nstg, Frame_lbl[Frm]);
		fprintf(stdout, "Results were written to %s.\n", Filew);
	}
	fclose(Fpw);
}

/* out_psfile -- write a PS file of the prsent plot */

void out_psfile(n)
int n;
{
	char fileps[100];

	while ((c=inpstr("PS file?",fileps,WFILE)) != ZERO){
		if (strcmp(fileps,FILEP) == 0){
			fprintf(stdout, "Name other than the reserved file \"%s\"!\n", FILEP);
			continue;
		}
		if (fopen(fileps,"r") != NULL)
			if (yorn("File exists. Overwrite? (Y or Rtn/N)",+1) != 1)
				continue;
		sys_call(4, "cp -f ", FILEP, " ", fileps);
		break;
	}
}
#undef CONTW0
#undef CONTW
