/* pdrc_dat -- read data from a file and stores to external variables Is[][], Ds[][], ...
	Internal variables:
		i -- counts number of sites
		j -- counts number of samples with linear regression result
		k -- counts number of samples with GTC analysis
		nmax -- maximum of linear data for a site
		ngmax -- maximum of GTC data for a site
*/

#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "pdirec.h"

#define CONTS0	"** Missing site locale of %s. Set to (0N,0E). **\n"
#define CONTS1	"** Missing site locale of %s. Set to previous site. **\n"

extern char **Site, **Smpl, **Smplg;
extern int Nsite, *Idsmp, *Idsmpg, *Id, *Rjc, *Idg, *Rjcg;
extern double *Slt, *Sln, **Is, **Ds, **Ip, **Dp, **Ia1, **Da1, **Ia2, **Da2, *Mad, *Madg;
extern double *Xs, *Ys, *Zs, *Dlt, *Dltg, *Xp, *Yp, *Zp, *Ig, *Dg, *Vlt, *Vln, *Vltr, *Vlnr, \
		*Vltg, *Vlng, *Vltgr, *Vlngr, *Dltv, *Dltvg;
extern double *Ieq, *Deq, *Ieqr, *Ieqg, *Deqg, *Ieqgr;

void pdrc_dat(fp, nline, ngtc)
FILE *fp;
int *nline, *ngtc;
{
	char s1[50],s2[50],s3[10],s4[10],s5[10],s6[10],s7[10],s8[10],s9[10],s10[10],s11[30];
	int nc, i = 0, j = 0, k = 0, jj = 0, kk = 0, nmax = 0, ngmax = 0, line1 = 0;
	double slatp = 0., slonp = 0.;

	while ((nc = get_s(fp, 11, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s11)) != EOF){
		if (nc == 0 || s1[0] == '#' || s1[0] == '%') continue;
		if (strcmp(s1, "$") == 0)
			i++;
		else {
			if (strcmp(s9, "P") == 0) k++;
			j++;
		}
	}
	Nsite = i;
	*nline = j - 3*k;   *ngtc = k;
	pdrc_mem1(Nsite, *nline, *ngtc);
	i = 0;   j = 0;   k = 0;   rewind(fp);
	while ((nc = get_s(fp, 11, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s11)) != EOF){
		if (nc == 0 || s1[0] == '#' || s1[0] == '%') continue;
		if (nc == 1) error("Error in data file!");
		if (line1 == 0){
			if (nc == 2){
				if (strcmp(s1, "$") == 0){
					Idsmp[i] = j;  Idsmpg[i] = k;  strcpy(Site[i], s2);
					Slt[i] = 0.;  Sln[i] = 0.;  slatp = 0.,  slonp = 0.;
					fprintf(stderr, CONTS0, s2);
					if (jj > nmax) nmax = jj;   jj = 0;
					if (kk > ngmax) ngmax = kk;   kk = 0;
					i++;
					line1 = 1;
					continue;
				}
				else
					error("Error in data file!");
			}
			if (nc == 3) error("Error in data file!");
			if (nc > 3) line1 = 1;
		}
		if (strcmp(s1, "$") == 0){
			Idsmp[i] = j;
			Idsmpg[i] = k;
			strcpy(Site[i], s2);
			if (nc == 4){
				Slt[i] = atof(s3);   Sln[i] = atof(s4);
				slatp = Slt[i];   slonp = Sln[i];
			}
			else {
				Slt[i] = slatp;   Sln[i] = slonp;
				fprintf(stderr, CONTS1, s2);
			}
			if (jj > nmax) nmax = jj;   jj = 0;
			if (kk > ngmax) ngmax = kk;   kk = 0;
			i++;
		}
		else {
			if (strcmp(s9, "P") == 0){
				strcpy(Smplg[k], s1);
				Ip[0][k] = atof(s2);   Dp[0][k] = atof(s3);
				Ia1[0][k] = atof(s4);   Da1[0][k] = atof(s5);
				Ia2[0][k] = atof(s6);   Da2[0][k] = atof(s7);
				Madg[k] = atof(s8);
				if (get_s(fp, 6, s1, s2, s3, s4, s5, s6) != 6)
					error("pdrc_dat: error at GTC data!");
				Ip[1][k] = atof(s1);   Dp[1][k] = atof(s2);
				Ia1[1][k] = atof(s3);   Da1[1][k] = atof(s4);
				Ia2[1][k] = atof(s5);   Da2[1][k] = atof(s6);
				if (get_s(fp, 6, s1, s2, s3, s4, s5, s6) != 6)
					error("pdrc_dat: error at GTC data!");
				Ip[2][k] = atof(s1);   Dp[2][k] = atof(s2);
				Ia1[2][k] = atof(s3);   Da1[2][k] = atof(s4);
				Ia2[2][k] = atof(s5);   Da2[2][k] = atof(s6);
				k++;   kk++;
			}
			else {
				strcpy(Smpl[j], s1);
				Is[0][j] = atof(s2);   Ds[0][j] = atof(s3);
				Is[1][j] = atof(s4);   Ds[1][j] = atof(s5);
				Is[2][j] = atof(s6);   Ds[2][j] = atof(s7);
				Mad[j] = atof(s8);
				j++;   jj++;
			}
		}
	}
	Idsmp[i] = j;
	Idsmpg[i] = k;
	if (jj > nmax) nmax = jj;
	if (kk > ngmax) ngmax = kk;
	pdrc_mem2(nmax, ngmax);
}

/* pdrc_mem1 -- memory alloc 1 for data variables which are related to all sites
	nsite, nsmpl, nsmplg are total no.s of site, sample, & GTC sample
*/

void pdrc_mem1(nsite, nsmpl, nsmplg)
int nsite, nsmpl, nsmplg;
{
	Site = mallocms(nsite, WSITE);
	Slt = mallocvd(nsite);   Sln = mallocvd(nsite);
	Idsmp = mallocvi(nsite+1);
	Idsmpg = mallocvi(nsite+1);

	Smpl = mallocms(nsmpl, WSAMP);
	Is = mallocmd(3, nsmpl);   Ds = mallocmd(3, nsmpl);
	Mad = mallocvd(nsmpl);

	Smplg = mallocms(nsmplg, WSAMP);
	Ip = mallocmd(3, nsmplg);   Dp = mallocmd(3, nsmplg);
	Ia1 = mallocmd(3, nsmplg);   Da1 = mallocmd(3, nsmplg);
	Ia2 = mallocmd(3, nsmplg);   Da2 = mallocmd(3, nsmplg);
	Madg = mallocvd(nsmplg);
}

/* pdrc_mem2 -- memory alloc 2 for variables which are related to the present site
	nmax, ngmax are maximum no.s of line & plane data
*/

void pdrc_mem2(nmax, ngmax)
int nmax, ngmax;
{
	Id = mallocvi(nmax);   Rjc = mallocvi(nmax);
	Xs = mallocvd(nmax);   Ys = mallocvd(nmax);  Zs = mallocvd(nmax);
	Dlt = mallocvd(nmax);   Dltv = mallocvd(nmax);
	Ieq = mallocvd(nmax);   Deq = mallocvd(nmax);   Ieqr = mallocvd(nmax);
	Vlt = mallocvd(nmax);   Vln = mallocvd(nmax);
	Vltr = mallocvd(nmax);   Vlnr = mallocvd(nmax);
	Idg = mallocvi(ngmax);   Rjcg = mallocvi(ngmax);
	Xp = mallocvd(ngmax);   Yp = mallocvd(ngmax);   Zp = mallocvd(ngmax);
	Dltg = mallocvd(ngmax);   Dltvg = mallocvd(nmax);
	Ig = mallocvd(ngmax);   Dg = mallocvd(ngmax);
	Ieqg = mallocvd(ngmax);   Deqg = mallocvd(ngmax);   Ieqgr = mallocvd(ngmax);
	Vltg = mallocvd(ngmax);   Vlng = mallocvd(ngmax);
	Vltgr = mallocvd(ngmax);   Vlngr = mallocvd(ngmax);
}

/* free_all -- free memory for all external array variables in reversed order
		-- DOES NOT WORK?
*/

void free_all(nsite, nsmpl, nsmplg)
int nsite, nsmpl, nsmplg;
{
	freevd(Vlngr);   freevd(Vltgr);
	freevd(Vlng);   freevd(Vltg);
	freevd(Ieqgr);   freevd(Deqg);   freevd(Ieqg);
	freevd(Dg);   freevd(Ig);
	freevd(Dltvg);   freevd(Dltg);
	freevd(Zp);   freevd(Yp);   freevd(Xp);
	freevi(Rjcg);   freevi(Idg);
	freevd(Vlnr);   freevd(Vltr);
	freevd(Vln);   freevd(Vlt);
	freevd(Ieqr);   freevd(Deq);   freevd(Ieq);
	freevd(Dltv);   freevd(Dlt);
	freevd(Zs);   freevd(Ys);   freevd(Xs);
	freevi(Rjc);   freevi(Id);

	freevd(Madg);
	freemd(3, Da2);   freemd(3, Ia2);
	freemd(3, Da1);   freemd(3, Ia1);
	freemd(3, Dp);   freemd(3, Ip);
	freems(nsmplg, Smplg);

	freevd(Mad);
	freemd(3, Ds);   freemd(3, Is);
	freems(nsmpl, Smpl);

	freevi(Idsmpg);
	freevi(Idsmp);
	freevd(Sln);   freevd(Slt);
	freems(nsite, Site);
}

/* free_all -- free memory for all external array variables -- DOES NOT WORK?

void free_all(nsite, nsmpl, nsmplg)
int nsite, nsmpl, nsmplg;
{
	freems(nsite, Site);
	freevd(Slt);   freevd(Sln);
	freevi(Idsmp);
	freevi(Idsmpg);

	freems(nsmpl, Smpl);
	freemd(3, Is);   freemd(3, Ds);
	freevd(Mad);

	freems(nsmplg, Smplg);
	freemd(3, Ip);   freemd(3, Dp);
	freemd(3, Ia1);   freemd(3, Da1);
	freemd(3, Ia2);   freemd(3, Da2);
	freevd(Madg);

	freevi(Id);   freevi(Rjc);
	freevd(Xs);   freevd(Ys);  freevd(Zs);
	freevd(Dlt);   freevd(Dltv);
	freevd(Ieq);   freevd(Deq);   freevd(Ieqr);
	freevd(Vlt);   freevd(Vln);
	freevd(Vltr);   freevd(Vlnr);
	freevi(Idg);   freevi(Rjcg);
	freevd(Xp);   freevd(Yp);   freevd(Zp);
	freevd(Dltg);   freevd(Dltvg);
	freevd(Ig);   freevd(Dg);
	freevd(Ieqg);   freevd(Deqg);   freevd(Ieqgr);
	freevd(Vltg);   freevd(Vlng);
	freevd(Vltgr);   freevd(Vlngr);
}
*/
#undef CONTS0
#undef CONTS1
