#include <string.h>
#include "pinte.h"

extern char **Sampl;
extern unsigned int Opt;
extern int Nd, Nstep, Nptr, Nmdtl, I1, I2;
extern double Scalax, Scalaxc;

static int c, i1, i2;

/* pi_comd -- returns a first letter of the hitted command
	normalize_trm() with command M was added   072514
	do_circle() with command C was added   092613
	chi_or_zjt() with command C was replaced by zjt_or_chi() with command Z
*/

int pi_comd(rem)
char *rem;
{
	fprintf(stdout, "%s ", rem);
	c = get_ch();
	if (c == ZERO) return(-1);
	else if (c == 'q' || c == 'Q') return ZERO;
	else return(c);
}

/* do_init -- called only when the sample is changed */

void do_init(n)
int n;
{
	fprintf(stdout, "%s\n", Sampl[n]);
	Nstep = pi_ntp(n, &Nptr, &Nmdtl, &I1, &I2);
	pi_gmtdat(n);
	Opt |= PLOT_CHI_BIT;
	pi_plot(n);
}

/* go_next */

int go_next(n)
int *n;
{
	if (*n >= Nd -1){
		fprintf(stdout, "End of file!\n");
		return ZERO;
	}
	*n += 1;
	do_init(*n);
}

/* go_back -- go back the previous sample */

int go_back(n)
int *n;
{
	if (*n <= 0){
		fprintf(stdout, "Beginning of file!\n");
		return ZERO;
	}
	*n -= 1;
	do_init(*n);
}

/* go_first */

void go_first(n)
int *n;
{
	*n = 0;
	do_init(*n);
}

/* go_last */

void go_last(n)
int *n;
{
	*n = Nd - 1;
	do_init(*n);
}

/* do_regres -- linear regression */

int do_regres(n)
int n;
{
	if (Opt & CIRCLE_BIT)
		Opt &= ~CIRCLE_BIT;
	fprintf(stdout, "%2d-%2d: ", 0, Nstep-1);
	while ((c=inpi("i1,i2? ",2,&i1,&i2)) != ZERO){
		if (c == -1) continue;
		if (i1>=0 && i2<Nstep && i2-i1>=2) break;
		else fprintf(stderr, "Invalid i1 & i2!\n");
	}
	if (c == ZERO) return ZERO;
	I1 = i1;   I2 = i2;   Opt |= REGRES_BIT;
	pi_regres(n);
	pi_plot(n);
}

/* do_circle -- circle regression */

int do_circle(n)
int n;
{
	if (Opt & REGRES_BIT)
		Opt &= ~REGRES_BIT;
	fprintf(stdout, "%2d-%2d: ", 0, Nstep-1);
	while ((c=inpi("i1,i2? ",2,&i1,&i2)) != ZERO){
		if (c == -1) continue;
		if (i1>=0 && i2<Nstep && i2-i1>=2) break;
		else fprintf(stderr, "Invalid i1 & i2!\n");
	}
	if (c == ZERO) return ZERO;
	I1 = i1;   I2 = i2;   Opt |= CIRCLE_BIT;
	pi_regres(n);
	pi_plot(n);
}

/* change_scale -- effective only for the present sample */

int change_scale(n)
int n;
{
	double d;

	while ((c=inpd("Scale to unity of NRM? ",1,&d)) != ZERO){
		if (c == -1) continue;
		if (d < 0.001 || d > 1000.) continue;
		Scalax = d;   Scalaxc = d;
		break;
	}
	if (c == ZERO) return ZERO;
	pi_plot(n);
}

/* normalize_trm -- ON & OFF of normalization of TRM by Maximum TRM */

void normalize_trm(n)
int n;
{
	if (Opt & NORM_TRM_BIT){
		Opt &= ~NORM_TRM_BIT;
		fprintf(stdout, "Normalization of TRM was restored.\n");
	}
	else {
		Opt |= NORM_TRM_BIT;
		fprintf(stdout, "TRM is now normalized to unity.\n");
	}
	Opt &= ~REGRES_BIT;
	Opt &= ~CIRCLE_BIT;
	I1 = 0;   I2 = Nstep-1;
	pi_plot(n);
}

/* change_orient -- change orientation from core to in situ, or vice versa */

void change_orient(n)
int n;
{
	if (Opt & IN_SITU_BIT)
		Opt &= ~IN_SITU_BIT;
	else
		Opt |= IN_SITU_BIT;
	I1 = 0;   I2 = Nstep-1;
	pi_gmtdat(n);
	pi_plot(n);
}

/* change_anchor -- ON & OFF of anchored regression line on Zj-plot of NRM steps */

void change_anchor(n)
int n;
{
	if (!(Opt & REGRES_BIT) && !(Opt & CIRCLE_BIT))
		fprintf(stdout, "First analyze by \'L\' or \'C\' commands!\n");
	else {
		if (Opt & ANCHOR_BIT){
			Opt &= ~ANCHOR_BIT;
			fprintf(stdout, "Anchored remanence direction was restored.\n");
		}
		else {
			Opt |= ANCHOR_BIT;
			fprintf(stdout, "Remanence direction is anchored to the origin.\n");
		}
		pi_regres(n);
		pi_plot(n);
	}
}

/* disable_annot -- change annotation from Temp. to No., or vice versa,
		 each time called.  works only for the present sample
*/

void disable_annot(n)
int n;
{
	if (Opt & TEMP_BIT){
		Opt &= ~TEMP_BIT;
		Opt |= NUMBER_BIT;
	}
	else {
		Opt &= ~NUMBER_BIT;
		Opt |= TEMP_BIT;
	}
	pi_plot(n);
}

/* change_ptlin -- delete or restore lines of pTRM test */

void change_ptlin(n)
int n;
{
	if (Opt & PTR_NOLINE_BIT)
		Opt &= ~PTR_NOLINE_BIT;
	else
		Opt |= PTR_NOLINE_BIT;
	pi_plot(n);
}

/* zjt_or_chi -- change over to Zjderveld for TRM steps from Chi-Step curve */

void zjt_or_chi(n)
int n;
{
	if (Opt & PLOT_CHI_BIT)
		Opt &= ~PLOT_CHI_BIT;
	else
		Opt |= PLOT_CHI_BIT;
	pi_plot(n);
}

/* out_psfile -- copy the present plot to a specified file */

void out_psfile(n)
int n;
{
	char fileps[100];

	while ((c=inpstr("PS file?",fileps,WFILE)) != ZERO){
		if (strcmp(fileps,FILEP) == 0){
			fprintf(stdout, "Name other than the reserved file \"%s\"!\n", FILEP);
			continue;
		}
		if (fopen(fileps,"r") != NULL)
			if (yorn("File exists. Overwrite? (Y or Rtn/N)",+1) != 1)
				continue;
		sys_call(4, "cp -f ", FILEP, " ", fileps);
		break;
	}
}
