#include <math.h>
#include "pinte.h"

#define MAX(a,b) ((a > b) ? a : b)
#define MIN(a,b) ((a < b) ? a : b)
#define EPS 1.e-20

extern unsigned int Opt;
extern int Nstep, Nptr, Nmdtl, *Idpt2, *Idmd, *Ia;
extern double *Incn, *Decn, *Incmd, *Decmd, *Inct, *Dect, *Incp, *Decp, *Xn, *Yn, \
		*Zn, *Xmd, *Ymd, *Zmd, *Xt, *Yt, *Zt, *Xp, *Yp, *Zp, *Magn, *Magmd, \
		*Magt, *Magp, *Chin, *Chit, *Chip, *Chim, *Azim, *Plun;
extern double *Ienn, *Denn, *Ienr, *Iemn, *Demn, *Iemr, *Ietn, *Detn, *Ietr, *Iepn, \
		*Depn, *Iepr, Xzn1, Xzn2, Yzn1, Yzn2, Gapzn, dXzn, *Xzn, *Yzn, *Zzn, \
		*Xzm, *Yzm, *Zzm, Xzt1, Xzt2, Yzt1, Yzt2, Gapzt, dXzt, *Xzt, *Yzt, *Zzt, \
		*Xzp, *Yzp, *Zzp, *Ycn, *Yct, *Ycp, *Ycm, Ycmax, Ycmin, dYca, dYct, \
		*Xa, *Xac, *Xam, *Xacm, *Ya, *Yam, *Xap, *Xacp, *Yap, Scala, Scalax, \
		Scalaxc, NrmbarTmax, Factzjn, *Incs, *Decs, *Xs, *Ys, *Zs; 

/* pi_gmtdat -- set up all the necessary GMT plot data for nth sample */

void pi_gmtdat(n)
int n;
{
	int i;
	double factzjt, set_zjdata(), xmax, ymax, yamax, yammax, magtmax, \
		dum, set_scalax();

	/* direction data for equal area plot */

	if ( !(Opt & IN_SITU_BIT) ){
		set_eqdata(Nstep, Incn, Decn, Ienn, Denn, Ienr);
		set_eqdata(Nmdtl, Incmd, Decmd, Iemn, Demn, Iemr);
		set_eqdata(Nstep, Inct, Dect, Ietn, Detn, Ietr);
		set_eqdata(Nptr, Incp, Decp, Iepn, Depn, Iepr);
	}
	else {
		set_insitu_id(n, Nstep, Incn, Decn, Incs, Decs);
		set_eqdata(Nstep, Incs, Decs, Ienn, Denn, Ienr);
		set_insitu_id(n, Nmdtl, Incmd, Decmd, Incs, Decs);
		set_eqdata(Nmdtl, Incs, Decs, Iemn, Demn, Iemr);
		set_insitu_id(n, Nstep, Inct, Dect, Incs, Decs);
		set_eqdata(Nstep, Incs, Decs, Ietn, Detn, Ietr);
		set_insitu_id(n, Nptr, Incp, Decp, Incs, Decs);
		set_eqdata(Nptr, Incs, Decs, Iepn, Depn, Iepr);
	}

	/* orthogonal data for Zijderveld plot */

	if ( !(Opt & IN_SITU_BIT) ){
		Factzjn = set_zjdata(Nstep, Xn, Yn, Zn, &Xzn1, &Xzn2, &Yzn1, &Yzn2, &Gapzn, \
			&dXzn, Xzn, Yzn, Zzn);
		for (i = 0; i < Nmdtl; i++){
			Xzm[i] = Xmd[i]*Factzjn;  Yzm[i] = Ymd[i]*Factzjn;  Zzm[i] = Zmd[i]*Factzjn;
		}
		factzjt = set_zjdata(Nstep, Xt, Yt, Zt, &Xzt1, &Xzt2, &Yzt1, &Yzt2, &Gapzt, \
			&dXzt, Xzt, Yzt, Zzt);
        	for (i = 0; i < Nptr; i++){
                	Xzp[i] = Xp[i]*factzjt;   Yzp[i] = Yp[i]*factzjt;   Zzp[i] = Zp[i]*factzjt;
        	}
	}
	else {
		set_insitu_xyz(n, Nstep, Xn, Yn, Zn, Xs, Ys, Zs);
		Factzjn = set_zjdata(Nstep, Xs, Ys, Zs, &Xzn1, &Xzn2, &Yzn1, &Yzn2, &Gapzn, \
			&dXzn, Xzn, Yzn, Zzn);
		set_insitu_xyz(n, Nmdtl, Xmd, Ymd, Zmd, Xs, Ys, Zs);
		for (i = 0; i < Nmdtl; i++){
			Xzm[i] = Xs[i]*Factzjn;  Yzm[i] = Ys[i]*Factzjn;  Zzm[i] = Zs[i]*Factzjn;
		}
		set_insitu_xyz(n, Nstep, Xt, Yt, Zt, Xs, Ys, Zs);
		factzjt = set_zjdata(Nstep, Xs, Ys, Zs, &Xzt1, &Xzt2, &Yzt1, &Yzt2, &Gapzt, \
			&dXzt, Xzt, Yzt, Zzt);
		set_insitu_xyz(n, Nptr, Xp, Yp, Zp, Xs, Ys, Zs);
        	for (i = 0; i < Nptr; i++){
                	Xzp[i] = Xs[i]*factzjt;   Yzp[i] = Ys[i]*factzjt;   Zzp[i] = Zs[i]*factzjt;
        	}
	}

	/* if Chin[0]!=0, normalized chi dat is set and READ_CHI_BIT is ON */

	set_chidat();
	
	/* normalized data for Arai plot */

	maxmin(Nstep, Magt, &magtmax, &dum);   NrmbarTmax = Magn[0]/magtmax;
        for (i = 0; i < Nstep; i++){
                Xa[i] = Magt[i]/Magn[0];   Xac[i] = Magt[i]/magtmax;
		Ya[i] = Magn[i]/Magn[0];   Ia[i] = i;
        }
        for (i = 0; i < Nmdtl; i++){
                Xam[i] = Xa[Idmd[i]];   Xacm[i] = Xac[Idmd[i]];
		Yam[i] = Magmd[i]/Magn[0];
	}
        for (i = 0; i < Nptr; i++){
                Xap[i] = Magp[i]/Magn[0];   Xacp[i] = Magp[i]/magtmax;
		Yap[i] = Ya[Idpt2[i]];
        }
        maxmin(Nstep, Ya, &yamax, &dum);   ymax = yamax;
	if (Nmdtl > 0){
		maxmin(Nmdtl, Yam, &yammax, &dum);
		ymax = MAX(yamax, yammax);
	}
        if (ymax >= YAMAX) Scala = ymax*YAMAX;
	else Scala = 1.;
	maxmin(Nstep, Xa, &xmax, &dum);   Scalax = set_scalax(xmax);
	maxmin(Nstep, Xac, &xmax, &dum);   Scalaxc = set_scalax(xmax);
	Opt |= TEMP_BIT;
	Opt &= ~REGRES_BIT;
	Opt &= ~CIRCLE_BIT;
	Opt &= ~NORM_TRM_BIT;
	Opt &= ~ANCHOR_BIT;
	Opt &= ~PTR_NOLINE_BIT;
}

/* set_eqdata */

void set_eqdata(n, inc, dec, incn, decn, incr)
int n;
double *inc, *dec, *incn, *decn, *incr;
{
	int i;

	for (i = 0; i < n; i++){
		incn[i] = inc[i];   decn[i] = 360. - dec[i];
		incr[i] = -inc[i];
	}
}
 
/* set_zjdata */

double set_zjdata(n, x, y, z, x1, x2, y1, y2, gap, dx, xz, yz, zz)
int n;
double *x, *y, *z, *x1, *x2, *y1, *y2, *gap, *dx, *xz, *yz, *zz;
{
	int i;
	double xmin, xmax, ymin, ymax, zmin, zmax, Xmin, Xmax, Ymin, Ymax, \
		Xc, Yc, width, height, length, order, factr, facte = 1.2, axmax, \
		d1, d2, set_dX();

	maxmin(n, x, &xmax, &xmin);
	maxmin(n, y, &ymax, &ymin);
	maxmin(n, z, &zmax, &zmin);
	Xmax = xmax;
	if (Xmax < 0.) Xmax = 0.;
	Xmin = xmin;
	if (Xmin > 0.) Xmin = 0.;
	Ymax = MAX(ymax, zmax);
	if (Ymax < 0.) Ymax = 0.;
	Ymin = MIN(ymin, zmin);
	if (Ymin > 0.) Ymin = 0.;
	width = Xmax - Xmin;   Xc = (Xmax + Xmin)/2.;
	height = Ymax - Ymin;   Yc = (Ymax + Ymin)/2.;
	if (width >= height){
		length = width;
		order = floor(log10(length));
		factr = pow(10., -order);
		*x1 = (Xc - (Xc - Xmin)*facte)*factr;
		*x2 = (Xc + (Xmax - Xc)*facte)*factr;
		*y2 = (Yc + 0.5*length*facte)*factr;
		*y1 = (Yc - 0.5*length*facte)*factr;
	}
	else {
		length = height;
		order = floor(log10(length));
		factr = pow(10., -order);
		*x1 = (Xc - 0.5*length*facte)*factr;
		*x2 = (Xc + 0.5*length*facte)*factr;
		*y2 = (Yc + (Ymax - Yc)*facte)*factr;
		*y1 = (Yc - (Yc - Ymin)*facte)*factr;
	}
	d1 = MAX(fabs(*x1), fabs(*x2));   d2 = MAX(fabs(*y1), fabs(*y2));
	axmax = MAX(d1, d2);
	*dx = set_dX(axmax);
	*gap = 0.01*length*factr;
	for (i = 0; i < n; i++){
		xz[i] = x[i]*factr;   yz[i] = y[i]*factr;   zz[i] = z[i]*factr;
	}
	return(factr);
}

/* set_dX -- used in set_zjdata() for Zijderveld diagram */

double set_dX(axmax)
double axmax;
{
        if (axmax < 0.8) return(0.5);
        if (axmax < 1.8) return(1.);
        if (axmax < 3.8) return(2.);
        else return(5.);
}

/* set_chidat -- set normalized chi data for chi-step curve and READ_CHI_BIT ON.
	upper level progs should set PLOT_CHI_BIT ON or OFF.   051919
*/

void set_chidat()
{
	int i;
	double ymax, ymin, dmax1, dmin1, dmax2, dmin2;

	if (Chin[0] < EPS){
		Opt &= ~READ_CHI_BIT;  Opt &= ~PLOT_CHI_BIT;
		return;
	}
	else {
		Opt |= READ_CHI_BIT;
	}
	for (i = 0; i < Nstep; i++){
		Ycn[i] = Chin[i]/Chin[0];   Yct[i] = Chit[i]/Chin[0];
	}
	if (Nptr > 0)
		for (i = 0; i < Nptr; i++) Ycp[i] = Chip[i]/Chin[0];
	if (Nmdtl > 0)
		for (i = 0; i < Nmdtl; i++) Ycm[i] = Chim[i]/Chin[0];
	maxmin(Nstep, Ycn, &dmax1, &dmin1);   maxmin(Nstep-1, Yct+1, &dmax2, &dmin2);
	ymax = MAX(dmax1, dmax2);   ymin = MIN(dmin1, dmin2);
	if (Nptr > 0){
		maxmin(Nptr, Ycp, &dmax1, &dmin1);
		ymax = MAX(ymax, dmax1);   ymin = MIN(ymin, dmin1);
	}
	if (Nmdtl > 0){
		maxmin(Nmdtl, Ycm, &dmax1, &dmin1);
		ymax = MAX(ymax, dmax1);   ymin = MIN(ymin, dmin1);
	}
	if (ymax < 1.2 && ymin > 0.8){
		Ycmax = 1.2;   Ycmin = 0.8;   dYca = 0.2;   dYct = 0.1;
	}
	else if (ymax < 1.4 && ymin > 0.6){
		Ycmax = 1.4;   Ycmin = 0.6;   dYca = 0.4;   dYct = 0.1;
	}
	else if (ymax < 1.5 && ymin > 0.5){
		Ycmax = 1.5;   Ycmin = 0.5;   dYca = 0.5;   dYct = 0.1;
	}
	else if (ymax < 1.6 && ymin > 0.4){
		Ycmax = 1.6;   Ycmin = 0.4;   dYca = 0.6;   dYct = 0.2;
	}
	else if (ymax < 2.0){
		Ycmax = 2.0;   Ycmin = 0.0;   dYca = 0.5;   dYct = 0.5;
	}
	else if (ymax < 3.0){
		Ycmax = 3.0;   Ycmin = 0.0;   dYca = 1.0;   dYct = 1.0;
	}
	else {
		Ycmax = 5.0;   Ycmin = 0.0;   dYca = 1.0;   dYct = 1.0;
	}
}

/* set_scalax -- used in pi_gmtdat() for Arai plot */

double set_scalax(xmax)
double xmax;
{
        if (xmax < 0.3) return(0.2);
        if (xmax < 0.75) return(0.5);
        if (xmax < 1.5) return(1.0);
        if (xmax < 2.25) return(1.5);
        if (xmax < 3.0) return(2.0);
        if (xmax < 3.75) return(2.5);
        if (xmax < 4.5) return(3.0);
        if (xmax < 7.5) return(5.);
        if (xmax < 15.) return(10.);
        return(20.);
}

/* set_insitu_id -- convert m-dimension array (inc,dec) to in situ direction for nth sample */

void set_insitu_id(n, m, inc, dec, incs, decs)
int n, m;
double *inc, *dec, *incs, *decs;
{
	int i;
	double x0, y0, z0, x, y, z, dumm;

	for (i = 0; i < m; i++){
		magcar(1., inc[i], dec[i], &x0, &y0, &z0);
		core(Azim[n], Plun[n], x0, y0, z0, &x, &y, &z);
		carmag(x, y, z, &dumm, incs+i, decs+i);
	}
}

/* set_insitu_xyz -- convert m-dimension array of (x,y,z) to in situ vector for nth sample */

void set_insitu_xyz(n, m, x, y, z, xs, ys, zs)
int n, m;
double *x, *y, *z, *xs, *ys, *zs;
{
	int i;

	for (i = 0; i < m; i++)
		core(Azim[n], Plun[n], x[i], y[i], z[i], xs+i, ys+i, zs+i);
}
#undef MAX
#undef MIN
#undef EPS
