/* pst_cmd -- interpretation of argc & argv when pstat was made to run */

#include <ctype.h>
#include <string.h>
#include "pstat.h"

#define USAGE   "Usage: pstat [-H] file"
#define HELP   "HELP MESSAGE OF PSTAT\n\
  Fisher statistics of site means V.1.2, H. Tanaka (2025,2018)\n\
  Adapted to use GMT 6.3.0 (GMT 5.4.3) by Wessel et al. (2019)\n\n\
Usage:\n\
  pstat [-H] file\n\n\
Options:\n\
  H -- show this help message\n\n\
Commands during analysis:\n\
  \'+\'-next, \'-\'-back, \'0\'-first, \'9\'-last, lo\'C\'ality-on/off, map-lon\'G\'itude,\n\
  so\'U\'thern-hemi-on/off, \'A\'lph95-on/off, di\'V\'ide-ns-on/off, \'D\'elete-one, \'R\'estore-one,\n\
  a\'S\'d-pole, a\'X\'ial-dpf-on/off, \'P\'sfile, \'H\'elp, \'Q\'uit\n\n\
Input file:\n\
  Input data file should be the output file from \"pdirec\".\n\
  Any lines with 1st letter \"#\"/\"%\" at any position are comments.\n\n\
  $ group_name1\n\
    site1 I D a95 N R k vlat vlon dp dm Sw Swv slat slon N0 Ng FRAME\n\
    site2 I D a95 N R k vlat vlon dp dm Sw Swv slat slon N0 Ng FRAME\n\
    site3                   .\n\
                            .\n\
                            .\n\
  $ group_name2\n\
    site1 I D a95 N R k vlat vlon dp dm Sw Swv slat slon N0 Ng FRAME\n\
                            .\n\
                            .\n\n\
Output files:\n\
  1. Details of the results are automatically stored in \"group_name.PST\"\n\
    which is rewritten every time the statistics is recalculated.\n\
  2. \'P\' command create a PS file with user specified name.\n\
  3. \"PSTAT.PS\" is re-written every time a new group is read.\n\
(Q to quit)"

extern unsigned int Opt;

FILE *pst_cmd(argc, argv, file)
int argc;
char *argv[], *file;
{
	FILE *fp;
	char *s;
	int flagf = 0;

	if (argc < 2) error(USAGE);
	while (--argc > 0){
		if (*argv[argc] == '-'){
			s = argv[argc] + 1;
			switch (toupper(*s)){
			case 'H':
				showless(HELP);
				break;
			default:
				error("Unknown option!");
				break;
			}
		}
		else {
			if (flagf != 0) error("Assign only one file!");
			strcpy(file, argv[argc]);
			if ((fp = fopen(file, "r")) == NULL) error("Error of open file!");
			flagf = 1;
		}
	}
	if (flagf != 0) return(fp);
	else error("Error of open file!");
}
#undef USAGE
#undef HELP
