/* psv_cmd -- interpretation of argc & argv when pstat was made to run */

#include <ctype.h>
#include <string.h>
#include "psvar.h"

#define USAGE   "Usage: psvar [-H] file"
#define HELP   "HELP MESSAGE OF PSVAR:\n\
  Paleomag directions of lava sequences V.1.1, H. Tanaka (2025,2018)\n\
  Adapted to GMT 6.3.0 classic mode (GMT 5.4.3) by Wessel et al. (2019).\n\n\
Usage:\n\
  psvar [-H] file\n\n\
Options:\n\
  H -- show this help message.\n\n\
Commands during analysis:\n\
  \'+\'-next, \'-\'-back, \'0\'-first, \'9\'-last, lo\'C\'ality-on/off, map-lon\'G\'itude,\n\
  map-base-\'L\'atitude, so\'U\'thern-hemi-on/off, \'A\'lph95-on/off, \'M\'ean-on/off,\n\
  \'V\'gp-path-on/off, \'D\'el-pre/post-vgps, site-\'I\'d-on/off, a\'S\'d-pole,\n\
  \'W\'atson-beran, \'N\'um-iteration, \'P\'sfile, \'Z\'-gmt-stamp, \'H\'elp, \'Q\'uit\n\n\
Input file:\n\
  Input data file should be an output file from \"pdirec\".\n\
  Any lines with 1st letter \"#\"/\"%\" at any position are comments.\n\n\
  $ polarity_group_name1\n\
    site1 I D a95 N R k vlat vlon dp dm Sw Swv slat slon FRAME\n\
    site2 I D a95 N R k vlat vlon dp dm Sw Swv slat slon FRAME\n\
     .                      .\n\
     .                      .\n\
  $ polarity_group_name2\n\
    site14                  .\n\
    site15                  .\n\
     .                      .\n\
     .                      .\n\n\
Output files:\n\
  1. Paleomagnetic statistics are stored in \"polarity_group_name.PSV\"\n\
    which is re-written every time the PS plot is renewed.\n\
  2. P command create a PS file with user specified name.\n\
  3. \"PSVAR.PS\" is re-written every time a new group is read.\n\
  4. Serial correlation test by Watson & Beran 1967 is stored in\n\
    \"polarity_group_name.WB\". All of random L & sorted L are\n\
    stored in \"*.WB0\" & \"*.WB1\" with the original L at 1st line.\n\
(Q to quit)"

extern unsigned int Opt;

FILE *psv_cmd(argc, argv, file)
int argc;
char *argv[], *file;
{
	FILE *fp;
	char *s;
	int flagf = 0;

	if (argc < 2) error(USAGE);
	while (--argc > 0){
		if (*argv[argc] == '-'){
			s = argv[argc] + 1;
			switch (toupper(*s)){
			case 'H':
				showless(HELP);
				break;
			default:
				error("Unknown option!");
				break;
			}
		}
		else {
			if (flagf != 0) error("Assign only one file!");
			strcpy(file, argv[argc]);
			if ((fp = fopen(file, "r")) == NULL) error("Error of open file!");
			flagf = 1;
		}
	}
	if (flagf != 0) return(fp);
	else return fopenr("Input file?", file);
}
#undef USAGE
#undef HELP
