#include <string.h>
#include "psvar.h"

extern char **Group;
extern unsigned int Opt;
extern int Ngrp, Nsite, Nwb;
extern double Slnm, Mapln, Mapltc, Nplt, Npln;

static int c;

/* psv_comd -- returns a first letter of the hitted command */

int psv_comd(rem)
char *rem;
{
	fprintf(stdout, "%s ", rem);
	c = get_ch();
	if (c == ZERO) return(-1);
	else if (c == 'q' || c == 'Q') return ZERO;
	else return(c);
}

/* do_init -- called when the group is changed */

void do_init(n)
int n;
{
	if (Opt & FIX_HEMI) Opt &= ~FIX_HEMI;
	if (Opt & WATSON_BERAN) Opt &= ~WATSON_BERAN;
	fprintf(stdout, "%s\n", Group[n]);
	psv_grpdat(n);
	psv_plot(n);
}

/* go_next */

void go_next(n)
int *n;
{
	if (*n >= Ngrp -1){
		fprintf(stdout, "End of file!\n");
		return;
	}
	*n += 1;
	do_init(*n);
}

/* go_back -- go back to the previous sample */

void go_back(n)
int *n;
{
	if (*n <= 0){
		fprintf(stdout, "Beginning of file!\n");
		return;
	}
	*n -= 1;
	do_init(*n);
}

/* go_first */

void go_first(n)
int *n;
{
	if (*n <= 0){
		fprintf(stdout, "Beginning of file!\n");
		return;
	}
	*n = 0;
	do_init(*n);
}

/* go_last */

void go_last(n)
int *n;
{
	if (*n >= Ngrp -1){
		fprintf(stdout, "End of file!\n");
		return;
	}
	*n = Ngrp - 1;
	do_init(*n);
}

/* plot_locality -- draw mean or distinctive localities */

void plot_locality(n)
int n;
{
	if ( !(Opt & SHOW_SITE) ){
		Opt |= SHOW_SITE;
		Opt &= ~SHOW_DIST_SITE;
	}
	else if ( !(Opt & SHOW_DIST_SITE) )
		Opt |= SHOW_DIST_SITE;
	else {
		Opt &= ~SHOW_SITE;
		Opt &= ~SHOW_DIST_SITE;
	}
	psv_plot(n);
}

/* map_longitude -- set central longitude of equal area VGP map */

void map_longitude(n)
int n;
{
	if (Opt & MAP_LONG){
		Opt &= ~MAP_LONG;
		Mapln = Slnm;
		fprintf(stdout, "Longitude of map center was restored.\n");
	}
	else {
		while ((c=inpd("Longitude of center?",1,&Mapln)) != ZERO){
			if (c == -1) continue;
			else break;
		}
		if (c == ZERO) return;
		checklon(&Mapln, 0);
		Opt |= MAP_LONG;
	}
	psv_plot(n);
}

/* map_cutoff_lat -- set lowest latitude of world map (default: equator) */

int map_cutoff_lat(n)
int n;
{
	double dum;

	while ((c=inpd("0<=Latcut<=80? ",1,&dum)) != ZERO){
		if (c == -1) continue;
		if (dum>=0. && dum<=80.) break;
	}
	if (c == ZERO) return ZERO;
	Mapltc = dum;
	psv_plot(n);
}

/* change_south_hemi -- change over the hemisphere of a VGP map */

void change_south_hemi(n)
int n;
{
	if ( !(Opt & FIX_HEMI) )
		Opt |= FIX_HEMI;
	if ( !(Opt & SOUTH_HEMI) )
		Opt |= SOUTH_HEMI;
	else
		Opt &= ~SOUTH_HEMI;
	psv_plot(n);
}

/* alph_on_off -- ON/OFF for alph95 of each field & vgp (those for the means remains) */

void alph_on_off(n)
int n;
{
	if ( !(Opt & PLOT_ALPH) )
		Opt |= PLOT_ALPH;
	else
		Opt &= ~PLOT_ALPH;
	psv_plot(n);
}

/* plot_mean -- on & off the mean field & vgp */

void plot_mean(n)
int n;
{
	if (!(Opt & PLOT_MEAN))
		Opt |= PLOT_MEAN;
	else
		Opt &= ~PLOT_MEAN;
	psv_plot(n);
}

/* plot_path -- on & off the path of field and vgp variation */

void plot_path(n)
int n;
{
	if (!(Opt & PLOT_PATH))
		Opt |= PLOT_PATH;
	else
		Opt &= ~PLOT_PATH;
	psv_plot(n);
}

/* delete_pre_post -- on/off of previous & post directions & VGPs */

void delete_pre_post(n)
int n;
{
	if (!(Opt & SHOW_PRE_POST))
		Opt |= SHOW_PRE_POST;
	else
		Opt &= ~SHOW_PRE_POST;
	psv_plot(n);
}

/* plot_id -- series change of anottation in site_id, number, and off */

void plot_id(n)
int n;
{
	if (!(Opt & PLOT_SITEID) && !(Opt & PLOT_NUMBER))
		Opt |= PLOT_SITEID;
	else if (Opt & PLOT_SITEID){
		Opt &= ~PLOT_SITEID;
		Opt |= PLOT_NUMBER;
	}
	else if (Opt & PLOT_NUMBER){
		Opt &= ~PLOT_SITEID;
		Opt &= ~PLOT_NUMBER;
	}
	psv_plot(n);
}

/* asd_north_pole(n) -- set ON/OFF to calculate ASD around north pole (Nplt,Npln)
	which can ba a user specified apparent pole
*/

void asd_north_pole(n)
int n;
{
	if ( !(Opt & ASD_NPOLE) ){
		fprintf(stdout, "ASD around a Fixed North Pole (%4.1f,%5.1f).\n", Nplt, Npln);
		if (yorn("  Is this Fixed Pole OK? (Y or Rtn/N)", 1) == 0){
			while ((c=inpd("  lat & lon of the north pole?",2,&Nplt,&Npln)) != ZERO){
				if (c == -1) continue;
				else break;
			}
			if (c == ZERO) return;
		}
		Opt |= ASD_NPOLE;
	}
	else {
		fprintf(stdout, "ASD around the mean.\n");
		Opt &= ~ASD_NPOLE;
		Nplt = 90.;   Npln = 0.;
	}
	psv_statf(n);
	psv_plot(n);
}

/* do_WB(n) -- call psv_WB(n) to test serial correlation by Watson & Beran */

void do_WB(n)
int n;
{
	if ( !(Opt & WATSON_BERAN) ){
		Opt |= WATSON_BERAN;
		psv_WB(n);
	}
	else
		Opt &= ~WATSON_BERAN;
	psv_plot(n);
}

/* num_iteration(n) -- change number of iteration at psv_WB(n) */

void num_iteration(n)
int n;
{
	if ( (Opt & WATSON_BERAN) ){
		Opt &= ~WATSON_BERAN;
	}
	fprintf(stdout, "No. of iteration at Watson&Beran (now %d)?", Nwb);
	while ((c=inpi("",1,&Nwb)) != ZERO){
		if (c == -1) continue;
		if (Nwb < 1000){
			printf("Nwb should be >=1000! Nwb?");
			continue;
		}
		else break;
	}
	if (c == ZERO){ Nwb = 2500;  return; }
	do_WB(n);
}

/* out_psfile -- copy the present plot to a specified file */

void out_psfile(n)
int n;
{
	char fileps[100];

	while ((c=inpstr("PS file?",fileps,WFILE)) != ZERO){
		if (strcmp(fileps,FILEP) == 0){
			fprintf(stdout, "Name other than the reserved file \"%s\"!\n", FILEP);
			continue;
		}
		if (fopen(fileps,"r") != NULL)
			if (yorn("File exists. Overwrite? (Y or Rtn/N)",+1) != 1)
				continue;
		sys_call(4, "cp -f ", FILEP, " ", fileps);
		break;
	}
}

/* plot_stamp -- on & off the GMT time stamp */

void plot_stamp(n)
int n;
{
	if (!(Opt & PLOT_STAMP))
		Opt |= PLOT_STAMP;
	else
		Opt &= ~PLOT_STAMP;
	psv_plot(n);
}
