/* alph95 -- V.1.3: (n+1) vectors to close the circle NOT to use GMT -L option   091303
	create n vectors which make up a circle of alpha 95   8/17/98
		based on the old graphical version on EGR98 & MS-C (12/25/86,9/17/86)
	Note:
		inc -- mean inc (or plat)
		dec -- mean dec (or plon)
		a95 -- alph95 around the mean
		n   -- number of points which simulate the circle (n >= 8)
		inca -- pointer to array of inc (or plat) of the N+1 storage! (V.1.3)
		deca -- pointer to array of dec (or plon) of the N+1 storage! (V.1.3)
	Return:
		n -- when successful
		-1 -- when the original a95 is negative or zero 
		      (nothing is stored into the arrays of inca & deca)
*/

#include <math.h>
#include "std.h"

#define NMIN 8

int alph95(inc, dec, a95, n, inca, deca)
int n;
double inc, dec, a95, *inca, *deca;
{
	int i;
	double inc0, dec0, th0, cost, sint, a, cosa, sina, c, s, t, dt, t1, t2, th, ddeca, d1;

	if (n < NMIN) error("Error of n in alph95!");
	inc0 = inc/RAD;   dec0 = dec/RAD;
	if (a95 <= 0. || a95 >= 180.) return(-1);
	th0 = HPAI - inc0;   cost = cos(th0);   sint = sin(th0);
	a = a95/RAD;   cosa = cos(a);   sina = sin(a);
	c = cost*cosa;   s = sint*sina;
	if (cost == 0.){
		t1 = 0.;   t2 = DPAI;
	}
	else if ((d1 = cosa/cost) >= 1. || d1 <= -1.){
		t1 = 0.;   t2 = DPAI;
	}
	else {
		t1 = asin( sin(acos(d1))/sina );   t2 = DPAI - t1;
	}
	if (th0 > HPAI){
		t1 = PAI - t1;   t2 = DPAI - t1;
	}
	t = 0.;   dt = DPAI/n;
	for (i = 0; i <= n; i++){
		th = acos(c + s*cos(t));
		ddeca = asin( sina*sin(t)/sin(th) );
		if (th0 <= HPAI){
			if (t < t1) ddeca = PAI - ddeca;
			else if (t > t2) ddeca = -PAI - ddeca;
		}
		else {
			if (t > t1 && t < PAI) ddeca = PAI - ddeca;
			else if (t > PAI && t < t2) ddeca = -PAI - ddeca;
		}
		*(inca+i) = RAD*(HPAI - th);   *(deca+i) = RAD*(dec0 + ddeca);
		checklon(deca+i, 0);
		t += dt;
	}
	return(n+1);
}
#undef NMIN
