#include <math.h>
#include "std.h"

/* dpm -- VDM (E+22 Am2) from F(micro T) and I(deg), Ver.2   0519,0290
		VDM = 2*pi*a^3*F*sqrt(1+3*cos^2 I)
	as F is given in mu T (F*10^-6 T) and VDM is given in 10^22 Am^2.
	calculate 2*pi*(6.3712x10^6)^3*F*10^-6/4*pi*10^-7/10^22, hence
		VDM = 0.129310*F*(1+3*cos^2 I)
	Arguments:
		dF: set negative when not available
		a95: usually set negative for not considered, otherwise give in degrees
		ddpm: return value of -1 when not available (dF<0 & a95<0)
*/

double dpm(F, inc, dF, a95, ddpm)
double F, inc, dF, a95, *ddpm;
{
	double cosi, sini, cosi2, sini2, c0=0.129310, c1, sqrc1;

	cosi = cos(inc/RAD);   cosi2 = cosi*cosi;
	c1 = 1. + 3.*cosi2;   sqrc1 = sqrt(c1);

	if (a95 < 0.){
		if (dF >= 0.) *ddpm = dF*c0*sqrc1;
		else *ddpm = -1.;
		return F*c0*sqrc1;
	}
	else {
		sini = sin(inc/RAD);   sini2 = sini*sini;
		a95 /= RAD;
		if (dF >= 0.)
			*ddpm = c0*sqrt(c1*dF*dF + 9.*F*F*cosi2*sini2*a95*a95/c1);
		else
			*ddpm = c0*3.*F*cosi*sini*a95/sqrc1;
		return F*c0*sqrc1;
	}
}

/* adpm -- virtual axial dipole moment from F and lat
		VADM = 4*pi*a^3*F/sqrt(a+3*sin^2 Lat)
	similarly to VDM, F and VADM are in 10^-6 T and 10^22 Am^2,
	with the value of 4*pi*(6.3712x10^6)^3*F*10^-6/4*pi*10^-7/10^22,
		VADM = 0.258621*F/sqrt(1+3*sin^2 Lat)
	Arguments:
		dF: set negative when not available
		dadpm: return value of -1 when not available (dF<0)
*/

double adpm(F, lat, dF, dadpm)
double F, lat, dF, *dadpm;
{
	double sinlt, c0 = 0.258621, m, dum;

	sinlt = sin(lat/RAD);
	dum = c0/sqrt(1. + 3.*sinlt*sinlt);
	m = F*dum;
	if (dF >= 0.) *dadpm = dF*dum;
	else *dadpm = -1.;
	return(m);
}
/*
	m = c0*F/sqrt(1. + 3.*sinlt*sinlt);
	if (dF >= 0.) *dadpm = m*dF/F;
*/
