#include <math.h>
#include "std.h"

/* fldadf -- convert (inc,dec) to (incp,decp) in axial dipole field coordinate under 
	apparent north pole (nplat,nplon).  This function supposes that the rotation axis 
	is inclined, while fldadp() always uses geographical NP   101103
	What actually do is to rotate (inc,dec) so that (incd,decd) comes to the center 
	downward which is the axial DP field under the apparent north pole
	procedure:
		0. obtain (incd,decd) for the site by vgpadf()
		1. rotate a vector 90-decd around z toward y, 
		2. rotate 90-incd around x toward z,
		3. rotate a vector -90 around z so that x becomes azimuth of DP field
*/

void fldadf(slat, slon, nplat, nplon, inc, dec, incp, decp)
double slat, slon, nplat, nplon, inc, dec, *incp, *decp;
{
	double incd, decd, dum1, dum2;

	vgpadf(slat, slon, nplat, nplon, &incd, &decd);
	rotuvec(90.-decd, 90.-incd, inc, dec, &dum1, &dum2);
	rotuvec(-90., 0., dum1, dum2, incp, decp);
}


/* vgpadf -- ADF direction at (slat,slon) under apparent north pole (nplat,nplon) 
	actually the same program as vgpfld() except the latter gives moment also 
*/

void vgpadf(slat, slon, nplat, nplon, incd, decd)
double slat, slon, nplat, nplon, *incd, *decd;
{
	double theta, g10, g11, h11, nplatr, nplonr, slatr, slonr, cosplt, 
		costh, sinth, cossln, sinsln, x, y, z, dumm;

	nplatr = nplat/RAD;   nplonr = nplon/RAD;
	cosplt = cos(nplatr);
	g10 = -sin(nplatr);
	g11 = -cosplt*cos(nplonr);
	h11 = -cosplt*sin(nplonr);
	slatr = slat/RAD;   slonr = slon/RAD;
	theta = HPAI - slatr;
	costh = cos(theta);   sinth = sin(theta);
	cossln = cos(slonr);   sinsln = sin(slonr);
	x = -g10*sinth + (g11*cossln + h11*sinsln)*costh;
	y = g11*sinsln - h11*cossln;
	z = -2.*g10*costh - 2.*(g11*cossln + h11*sinsln)*sinth;
	carmag(x, y, z, &dumm, incd, decd);
}
