#include <math.h>
#include "std.h"

/* hista -- automatically count number of data within each bin for a given bin number   063099
	(xi <= x < xi+1, however, for the largest bin, xmax-1 <= x <= xmax)
*/

void hista(n, x, nbin, bins)
int n, nbin;
double *x, *bins;
{
	int i, j;
	double xmax, xmin, dx;

	if (n < 1 || nbin < 1) error("Illegal number of data or bins!");
	for (i = 0; i < nbin; i++) bins[i] = 0.;
	maxmin(n, x, &xmax, &xmin);
	dx = (xmax - xmin)/nbin;
	if (dx <= 0.){
		bins[0] = n;
		return;
	}
	for (i = 0; i < n; i++){
		j = floor((x[i] - xmin)/dx);
		if (j < 0) j = 0;
		if (j >= nbin) j = nbin - 1;
		bins[j] += 1.;
	}
}

/* histd -- count number of data within each bin starting xmin with indterval dx 
	   (count frequency of data as xi <= x < xi+dx, exception is the data just at
	    the right edge of the largest bin which is counted to the last bin)
*/

void histd(n, x, xmin, dx, nbinmax, bins)
int n, nbinmax;
double *x, xmin, dx, *bins;
{
	int i, j;

	if (n < 1 || nbinmax < 1) error("Illegal number of data or bins!");
	for (i = 0; i < nbinmax; i++) bins[i] = 0.;
	for (i = 0; i < n; i++){
		j = floor((x[i] - xmin)/dx);
		if (j >= 0 && j < nbinmax) bins[j] += 1.;
		if (j == nbinmax) bins[nbinmax-1] += 1.;
	}
}

/* hist2c -- count data numbers of two data x1[] & x2[] in bin1[] & bin2[] which are
	devided with common boundary xb[]. Boundaries are determined so that cumulative
	distribution of the combined data are evenly divided.
*/

void hist2c(n1, n2, x1, x2, nbin, bin1, bin2)
int n1, n2, nbin;
double *x1, *x2, *bin1, *bin2;
{
	int n, i;
	double *xb, *x;

	n = n1 + n2;
	x = mallocvd(n);   xb = mallocvd(nbin-1);
	for (i=0; i<n1; i++) x[i] = x1[i];
	for (i=0; i<n2; i++) x[n1+i] = x2[i];
	for (i=0; i<nbin; i++){ bin1[i] = 0.;   bin2[i] = 0.; }
	binboundc(n, x, nbin, xb);
	countbin(n1, x1, nbin, xb, bin1);
	countbin(n2, x2, nbin, xb, bin2);
	freevd(x);   freevd(xb);
}

/* binboundc -- determine x value of bin boundaries xb[] which evenly divide the cumulative
	distribution of x[]. It is supposed that x[] are the combined data from two datasets
	which will be tested later whether they share a common distribution. Before the test
	by chi2test2, xb[] will be used in hist2c.
*/

void binboundc(n, x, nb, xb)
int n, nb;
double *x, *xb;
{
	int i, j;
	double nd, *xs, h, ib;

	nd = n;
	xs = mallocvd(n);
	for (i=0; i<n; i++) xs[i] = x[i];
	sort(n, xs);
	h = nd/nb;
	for (j=0; j<nb-1; j++){
		ib = h*(j+1) - 0.5;
		i = floor(ib);
		xb[j] = (xs[i+1] - xs[i])*(ib-i) + xs[i];
	}
	freevd(xs);
}

/* maxmin */

void maxmin(n, x, max, min)
int n;
double *x, *max, *min;
{
	int i;

	*max = x[0];   *min = x[0];
	for (i = 0; i < n; i++){
		if (x[i] > *max) *max = x[i];
		if (x[i] < *min) *min = x[i];
	}
}
/*
printf("h: %f\n", h);
printf("j,ib,i,xs[i],xb[j]: %2d %7.4f %2d %7.4f %7.4f\n", j, ib, i, xs[i], xb[j]);
*/
