/* meanvar -- modified from avevar.c of Numerical Recipes which seems superior to
	my meanstdv in terms of roundoff error.   092218
*/

#include <math.h>
#include "std.h"

void meanvar(n, x, mx, vx, sx)
int n;
double *x, *mx, *vx, *sx;
{
	int i;
	double nd, s, ep;

	if (n < 1) error("Error of n in meanvar!");
	if (n == 1){
		*mx = *x;   *vx = *sx = 0.;
		return;
	}
	nd = (double)n;
	for (*mx=0., i=0; i<n; i++) *mx += x[i];
	*mx /= nd;
	*vx = ep = 0.;
	for (i=0; i<n; i++) {
		s = x[i] - (*mx);
		ep += s;
		*vx += s*s;
	}
	*vx = (*vx - ep*ep/nd)/(nd - 1.);
	*sx = sqrt(*vx);
}

/* meanvarl -- given positive x[i], returns mean, var, stdev of y=log(x) (x=e^y),
	supposing y[i] is normally distributed, i.e. x[i] is log-normal.
		my, vy, sy -- mean, var, stdev of y=log(x) (x=e^y)
		mx, vx -- mean & var of x following log-normal distribution
			mx=E[x]=e^(my+vy/2)
			vx=E[x^2]-mx^2=e^(2*my+vy)(e^vy-1)                112918
*/

void meanvarl(n, x, my, vy, sy, mx, vx)
int n;
double *x, *my, *vy, *sy, *mx, *vx;
{
	int i;
	double *y;

	if (n < 1) error("Error of n in meanvarl!");
	if (n == 1){
		if (x[0] <= 0.) error("Error of x<=0 in meanvarl!");
		*my = log(x[0]);  *mx = x[0];  *vy = *sy = *vx = 0.;
		return;
	}
	y = mallocvd(n);
	for (i=0; i<n; i++){
		if (x[i] <= 0.) error("Error of x<=0 in meanvarl!");
		y[i] = log(x[i]);
	}
	meanvar(n, y, my, vy, sy);
	*mx = exp(*my + *vy/2.);
	*vx = exp(2.*(*my) + *vy)*(exp(*vy) - 1.);
	freevd(y);
}
