#include <math.h>
#include "std.h"

#define EPS 1.e-8

static double pp;

/* normtab -- percentile of the Normal distribution for F(z), ie, returns z at which 
	the cumulative distribution F(z), integral of f(z) from -oo to z, is p.
	find a root of the next equation by calling rtsafe()   120898
		f(z) = p - normcdist(z) = 0
*/

double normtab(p)
double p;
{
	void f_norm();
	double rl = -5.0, rh = .0001;

	if (p < 0.00001 || p > .99999) error("normtab: p should be 0.00001 <= p <= .99999!");
	if (p < 0.5){
		pp = p;
		return rtsafe(f_norm, rl, rh, EPS);
	}
	else {
		pp = 1. - p;
		return -rtsafe(f_norm, rl, rh, EPS);
	}
}


/* normcdist -- cumulative distribution of the Normal distribution for z, ie, 
	returns F(z), integral of f(z) from -oo to z.
*/

double normcdist(z)
double z;
{
	return (z < 0.) ? 1. - erfnc(z/RT2)/2. : .5 + erfn(z/RT2)/2.;
}


/* f_norm -- function to determine the root for normtab */

void f_norm(x, f, df)
double x, *f, *df;
{
	*f = pp - normcdist(x);
	*df = -exp(-x*x/2.)/sqrt(2.*PAI);
}


/* normdist -- density function of normal distribution with zero mean & variance of 1 */

double normdist(z)
double z;
{
	return exp(-z*z/2.)/sqrt(2.*PAI);
}
#undef EPS
